@@ -2,28 +2,36 @@ package commands
 
 import (
 	"cmp"
+	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"os"
+	"regexp"
+	"strconv"
 	"strings"
 	"sync"
+	"sync/atomic"
+	"time"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/bindserial"
 	"github.com/StackExchange/dnscontrol/v4/pkg/credsfile"
+	"github.com/StackExchange/dnscontrol/v4/pkg/domaintags"
 	"github.com/StackExchange/dnscontrol/v4/pkg/nameservers"
 	"github.com/StackExchange/dnscontrol/v4/pkg/normalize"
 	"github.com/StackExchange/dnscontrol/v4/pkg/notifications"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	"github.com/StackExchange/dnscontrol/v4/pkg/rfc4183"
 	"github.com/StackExchange/dnscontrol/v4/pkg/zonerecs"
-	"github.com/StackExchange/dnscontrol/v4/providers"
-	"github.com/urfave/cli/v2"
+	"github.com/nozzle/throttler"
+	"github.com/urfave/cli/v3"
 	"golang.org/x/exp/slices"
 	"golang.org/x/net/idna"
 )
 
-type zoneCache struct {
+type cmdZoneCache struct {
 	cache map[string]*[]string
 	sync.Mutex
 }
@@ -31,9 +39,9 @@ type zoneCache struct {
 var _ = cmd(catMain, func() *cli.Command {
 	var args PPreviewArgs
 	return &cli.Command{
-		Name:  "ppreview",
+		Name:  "preview",
 		Usage: "read live configuration and identify changes to be made, without applying them",
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, c *cli.Command) error {
 			return exit(PPreview(args))
 		},
 		Flags: args.flags(),
@@ -45,16 +53,18 @@ type PPreviewArgs struct {
 	GetDNSConfigArgs
 	GetCredentialsArgs
 	FilterArgs
-	Notify      bool
-	WarnChanges bool
-	ConcurMode  string
-	NoPopulate  bool
-	DePopulate  bool
-	Full        bool
+	Notify            bool
+	WarnChanges       bool
+	ConcurMode        string
+	ConcurMax         int // Maximum number of concurrent connections
+	NoPopulate        bool
+	PopulateOnPreview bool
+	Report            string
+	Full              bool
 }
 
 // ReportItem is a record of corrections for a particular domain/provider/registrar.
-//type ReportItem struct {
+// type ReportItem struct {
 //	Domain      string `json:"domain"`
 //	Corrections int    `json:"corrections"`
 //	Provider    string `json:"provider,omitempty"`
@@ -78,11 +88,25 @@ func (args *PPreviewArgs) flags() []cli.Flag {
 	flags = append(flags, &cli.StringFlag{
 		Name:        "cmode",
 		Destination: &args.ConcurMode,
-		Value:       "default",
-		Usage:       `Which providers to run concurrently: all, default, none`,
-		Action: func(c *cli.Context, s string) error {
-			if !slices.Contains([]string{"all", "default", "none"}, s) {
-				fmt.Printf("%q is not a valid option for --cmode.  Valie are: all, default, none", s)
+		Value:       "concurrent",
+		Usage:       `Which providers to run concurrently: concurrent, none, all`,
+		Action: func(ctx context.Context, c *cli.Command, s string) error {
+			if !slices.Contains([]string{"concurrent", "none", "all"}, s) {
+				fmt.Printf("%q is not a valid option for --cmode.  Values are: concurrent, none, all\n", s)
+				os.Exit(1)
+			}
+			return nil
+		},
+	})
+	flags = append(flags, &cli.IntFlag{
+		Name:        "cmax",
+		Destination: &args.ConcurMax,
+		Value:       999,
+		Usage:       `Maximum number of concurrent connections`,
+		Action: func(ctx context.Context, c *cli.Command, v int) error {
+			if v < 1 {
+				fmt.Printf("%d is not a valid value for --cmax.  Values must be 1 or greater\n", v)
+				os.Exit(1)
 			}
 			return nil
 		},
@@ -93,9 +117,10 @@ func (args *PPreviewArgs) flags() []cli.Flag {
 		Usage:       `Do not auto-create zones at the provider`,
 	})
 	flags = append(flags, &cli.BoolFlag{
-		Name:        "depopulate",
-		Destination: &args.NoPopulate,
-		Usage:       `Delete unknown zones at provider (dangerous!)`,
+		Name:        "populate-on-preview",
+		Destination: &args.PopulateOnPreview,
+		Value:       false,
+		Usage:       `Auto-create zones on preview`,
 	})
 	flags = append(flags, &cli.BoolFlag{
 		Name:        "full",
@@ -104,10 +129,10 @@ func (args *PPreviewArgs) flags() []cli.Flag {
 	})
 	flags = append(flags, &cli.IntFlag{
 		Name:   "reportmax",
-		Hidden: true,
+		Hidden: false,
 		Usage:  `Limit the IGNORE/NO_PURGE report to this many lines (Expermental. Will change in the future.)`,
-		Action: func(ctx *cli.Context, max int) error {
-			printer.MaxReport = max
+		Action: func(ctx context.Context, c *cli.Command, maxreport int) error {
+			printer.MaxReport = maxreport
 			return nil
 		},
 	})
@@ -116,15 +141,20 @@ func (args *PPreviewArgs) flags() []cli.Flag {
 		Destination: &bindserial.ForcedValue,
 		Usage:       `Force BIND serial numbers to this value (for reproducibility)`,
 	})
+	flags = append(flags, &cli.StringFlag{
+		Name:        "report",
+		Destination: &args.Report,
+		Usage:       `Generate a machine-parseable report of corrections.`,
+	})
 	return flags
 }
 
 var _ = cmd(catMain, func() *cli.Command {
 	var args PPushArgs
 	return &cli.Command{
-		Name:  "ppush",
+		Name:  "push",
 		Usage: "identify changes to be made, and perform them",
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, c *cli.Command) error {
 			return exit(PPush(args))
 		},
 		Flags: args.flags(),
@@ -135,7 +165,6 @@ var _ = cmd(catMain, func() *cli.Command {
 type PPushArgs struct {
 	PPreviewArgs
 	Interactive bool
-	Report      string
 }
 
 func (args *PPushArgs) flags() []cli.Flag {
@@ -145,17 +174,12 @@ func (args *PPushArgs) flags() []cli.Flag {
 		Destination: &args.Interactive,
 		Usage:       "Interactive. Confirm or Exclude each correction before they run",
 	})
-	flags = append(flags, &cli.StringFlag{
-		Name:        "report",
-		Destination: &args.Report,
-		Usage:       `Generate a machine-parseable report of performed corrections.`,
-	})
 	return flags
 }
 
 // PPreview implements the preview subcommand.
 func PPreview(args PPreviewArgs) error {
-	return prun(args, false, false, printer.DefaultPrinter, "")
+	return prun(args, false, false, printer.DefaultPrinter, args.Report)
 }
 
 // PPush implements the push subcommand.
@@ -167,7 +191,6 @@ var pobsoleteDiff2FlagUsed = false
 
 // run is the main routine common to preview/push
 func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, report string) error {
-
 	// This is a hack until we have the new printer replacement.
 	printer.SkinnyReport = !args.Full
 	fullMode := args.Full
@@ -182,14 +205,36 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 		return err
 	}
 
-	out.PrintfIf(fullMode, "Reading creds.json or equiv.\n")
+	out.PrintfIf(fullMode, "Reading creds: %q\n", args.CredsFile)
 	providerConfigs, err := credsfile.LoadProviderConfigs(args.CredsFile)
 	if err != nil {
 		return err
 	}
 
+	var notify = args.Notify
+
+	// We want to notify if args.Notify OR notify_on_*
+	if notifications, ok := providerConfigs["notifications"]; ok && notifications != nil {
+		if push {
+			if notifyOnPush, ok := notifications["notify_on_push"]; ok {
+				if b, _ := strconv.ParseBool(notifyOnPush); b {
+					notify = true
+				}
+			}
+		} else {
+			if notifyOnPreview, ok := notifications["notify_on_preview"]; ok {
+				if b, _ := strconv.ParseBool(notifyOnPreview); b {
+					notify = true
+				}
+			}
+		}
+	}
+	if notify {
+		out.PrintfIf(fullMode, "Notifications are enabled...\n")
+	}
+
 	out.PrintfIf(fullMode, "Creating an in-memory model of 'desired'...\n")
-	notifier, err := PInitializeProviders(cfg, providerConfigs, args.Notify)
+	notifier, err := PInitializeProviders(cfg, providerConfigs, notify)
 	if err != nil {
 		return err
 	}
@@ -197,41 +242,140 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 	out.PrintfIf(fullMode, "Normalizing and validating 'desired'..\n")
 	errs := normalize.ValidateAndNormalizeConfig(cfg)
 	if PrintValidationErrors(errs) {
-		return fmt.Errorf("exiting due to validation errors")
+		return errors.New("exiting due to validation errors")
 	}
 
-	zcache := NewZoneCache()
+	zcache := NewCmdZoneCache()
 
 	// Loop over all (or some) zones:
 	zonesToProcess := whichZonesToProcess(cfg.Domains, args.Domains)
 	zonesSerial, zonesConcurrent := splitConcurrent(zonesToProcess, args.ConcurMode)
-	out.PrintfIf(fullMode, "PHASE 1: GATHERING data\n")
-	var wg sync.WaitGroup
-	wg.Add(len(zonesConcurrent))
-	out.Printf("CONCURRENTLY gathering %d zone(s)\n", len(zonesConcurrent))
-	for _, zone := range optimizeOrder(zonesConcurrent) {
-		out.PrintfIf(fullMode, "Concurrently gathering: %q\n", zone.Name)
-		go func(zone *models.DomainConfig, args PPreviewArgs, zcache *zoneCache) {
-			defer wg.Done()
-			oneZone(zone, args, zcache)
-		}(zone, args, zcache)
-	}
-	out.Printf("SERIALLY gathering %d zone(s)\n", len(zonesSerial))
-	for _, zone := range zonesSerial {
-		out.Printf("Serially Gathering: %q\n", zone.Name)
-		oneZone(zone, args, zcache)
-	}
-	out.PrintfIf(len(zonesConcurrent) > 0, "Waiting for concurrent gathering(s) to complete...")
-	wg.Wait()
-	out.PrintfIf(len(zonesConcurrent) > 0, "DONE\n")
+	zonesConcurrent = optimizeOrder(zonesConcurrent)
 
-	// Now we know what to do, print or do the tasks.
-	out.PrintfIf(fullMode, "PHASE 2: CORRECTIONS\n")
 	var totalCorrections int
 	var reportItems []*ReportItem
 	var anyErrors bool
+	var concurrentErrors atomic.Bool
+
+	// Populate the zones (if desired/needed/able):
+	if !args.NoPopulate {
+		out.PrintfIf(fullMode, "PHASE 1: CHECKING for missing zones\n")
+		t := throttler.New(args.ConcurMax, len(zonesConcurrent))
+		out.Printf("CONCURRENTLY checking for %d zone(s)\n", len(zonesConcurrent))
+		for i, zone := range zonesConcurrent {
+			out.PrintfIf(fullMode, "Concurrently checking for zone: %q\n", zone.UniqueName)
+			go func(zone *models.DomainConfig) {
+				start := time.Now()
+				err := oneZonePopulate(zone, zcache)
+				if err != nil {
+					concurrentErrors.Store(true)
+				}
+				out.Debugf("...DONE: %q (%.1fs)\n", zone.Name, time.Since(start).Seconds())
+				t.Done(err)
+			}(zone)
+			// Delay the last call to t.Throttle() until the serial processing is done.
+			if i != ultimate(zonesConcurrent) {
+				errorCount := t.Throttle()
+				if errorCount > 0 {
+					anyErrors = true
+				}
+			}
+		}
+
+		out.Printf("SERIALLY checking for %d zone(s)\n", len(zonesSerial))
+		for _, zone := range zonesSerial {
+			out.Printf("Serially checking for zone: %q\n", zone.UniqueName)
+			if err := oneZonePopulate(zone, zcache); err != nil {
+				anyErrors = true
+			}
+		}
+
+		if len(zonesConcurrent) > 0 {
+			if printer.DefaultPrinter.Verbose {
+				out.PrintfIf(true, "Waiting for concurrent checking(s) to complete...\n")
+			} else {
+				out.PrintfIf(true, "Waiting for concurrent checking(s) to complete...")
+			}
+			errorCount := t.Throttle()
+			if errorCount > 0 {
+				anyErrors = true
+			}
+			out.PrintfIf(true, "DONE\n")
+		}
+
+		for _, zone := range zonesToProcess {
+			started := false // Do not emit noise when no provider has corrections.
+			providersToProcess := whichProvidersToProcess(zone.DNSProviderInstances, args.Providers)
+			for _, provider := range zone.DNSProviderInstances {
+				corrections := zone.GetPopulateCorrections(provider.Name)
+				if len(corrections) == 0 {
+					continue // Do not emit noise when zone exists
+				}
+				if !started {
+					out.StartDomain(zone)
+					started = true
+				}
+				skip := skipProvider(provider.Name, providersToProcess)
+				out.StartDNSProvider(provider.Name, skip)
+				if !skip {
+					totalCorrections += len(corrections)
+					out.EndProvider2(provider.Name, len(corrections))
+					reportItems = append(reportItems, genReportItem(zone.Name, corrections, provider.Name, ""))
+					anyErrors = cmp.Or(anyErrors, pprintOrRunCorrections(zone.Name, provider.Name, corrections, out, push || args.PopulateOnPreview, interactive, notifier, report))
+				}
+			}
+		}
+	}
+
+	out.PrintfIf(fullMode, "PHASE 2: GATHERING data\n")
+	t := throttler.New(args.ConcurMax, len(zonesConcurrent))
+	out.Printf("CONCURRENTLY gathering records of %d zone(s)\n", len(zonesConcurrent))
+	for i, zone := range zonesConcurrent {
+		out.PrintfIf(fullMode, "Concurrently gathering: %q\n", zone.UniqueName)
+		go func(zone *models.DomainConfig, args PPreviewArgs, zcache *cmdZoneCache) {
+			start := time.Now()
+			err := oneZone(zone, args)
+			if err != nil {
+				concurrentErrors.Store(true)
+			}
+			out.Debugf("...DONE: %q (%.1fs)\n", zone.Name, time.Since(start).Seconds())
+			t.Done(err)
+		}(zone, args, zcache)
+		// Delay the last call to t.Throttle() until the serial processing is done.
+		if i != ultimate(zonesConcurrent) {
+			errorCount := t.Throttle()
+			if errorCount > 0 {
+				anyErrors = true
+			}
+		}
+	}
+	out.Printf("SERIALLY gathering records of %d zone(s)\n", len(zonesSerial))
+	for _, zone := range zonesSerial {
+		out.Printf("Serially Gathering: %q\n", zone.UniqueName)
+		if err := oneZone(zone, args); err != nil {
+			anyErrors = true
+		}
+	}
+
+	if len(zonesConcurrent) > 0 {
+		msg := "Waiting for concurrent gathering(s) to complete..."
+		if printer.DefaultPrinter.Verbose {
+			msg = "Waiting for concurrent gathering(s) to complete...\n"
+		}
+		out.PrintfIf(true, msg)
+		errorCount := t.Throttle()
+		if errorCount > 0 {
+			anyErrors = true
+		}
+		out.PrintfIf(true, "DONE\n")
+	}
+
+	anyErrors = cmp.Or(anyErrors, concurrentErrors.Load())
+
+	// Now we know what to do, print or do the tasks.
+	out.PrintfIf(fullMode, "PHASE 3: CORRECTIONS\n")
 	for _, zone := range zonesToProcess {
-		out.StartDomain(zone.GetUniqueName())
+		out.StartDomain(zone)
 
 		// Process DNS provider changes:
 		providersToProcess := whichProvidersToProcess(zone.DNSProviderInstances, args.Providers)
@@ -240,10 +384,10 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 			out.StartDNSProvider(provider.Name, skip)
 			if !skip {
 				corrections := zone.GetCorrections(provider.Name)
-				numActions := countActions(corrections)
+				numActions := zone.GetChangeCount(provider.Name)
 				totalCorrections += numActions
 				out.EndProvider2(provider.Name, numActions)
-				reportItems = append(reportItems, genReportItem(zone.Name, corrections, provider.Name))
+				reportItems = append(reportItems, genReportItem(zone.Name, corrections, provider.Name, ""))
 				anyErrors = cmp.Or(anyErrors, pprintOrRunCorrections(zone.Name, provider.Name, corrections, out, push, interactive, notifier, report))
 			}
 		}
@@ -253,13 +397,12 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 		out.StartRegistrar(zone.RegistrarName, !skip)
 		if skip {
 			corrections := zone.GetCorrections(zone.RegistrarInstance.Name)
-			numActions := countActions(corrections)
+			numActions := zone.GetChangeCount(zone.RegistrarInstance.Name)
 			out.EndProvider2(zone.RegistrarName, numActions)
 			totalCorrections += numActions
-			reportItems = append(reportItems, genReportItem(zone.Name, corrections, zone.RegistrarName))
+			reportItems = append(reportItems, genReportItem(zone.Name, corrections, "", zone.RegistrarName))
 			anyErrors = cmp.Or(anyErrors, pprintOrRunCorrections(zone.Name, zone.RegistrarInstance.Name, corrections, out, push, interactive, notifier, report))
 		}
-
 	}
 
 	if os.Getenv("TEAMCITY_VERSION") != "" {
@@ -270,36 +413,27 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 	out.Printf("Done. %d corrections.\n", totalCorrections)
 	err = writeReport(report, reportItems)
 	if err != nil {
-		return fmt.Errorf("could not write report")
+		return errors.New("could not write report")
 	}
 	if anyErrors {
-		return fmt.Errorf("completed with errors")
+		return errors.New("completed with errors")
 	}
 	if totalCorrections != 0 && args.WarnChanges {
-		return fmt.Errorf("there are pending changes")
+		return errors.New("there are pending changes")
 	}
 	return nil
 }
 
-func countActions(corrections []*models.Correction) int {
-	r := 0
-	for _, c := range corrections {
-		if c.F != nil {
-			r++
-		}
-	}
-	return r
-}
-
+// whichZonesToProcess takes a list of DomainConfigs and a filter string and
+// returns a list of DomainConfigs whose Domain.UniqueName matched the
+// filter. The filter string is a comma-separated list of domain names. If the
+// filter string is empty or "all", all domains are returned.
 func whichZonesToProcess(domains []*models.DomainConfig, filter string) []*models.DomainConfig {
-	if filter == "" || filter == "all" {
-		return domains
-	}
+	fh := domaintags.CompilePermitList(filter)
 
-	permitList := strings.Split(filter, ",")
 	var picked []*models.DomainConfig
 	for _, domain := range domains {
-		if domainInList(domain.Name, permitList) {
+		if fh.Permitted(domain.GetUniqueName()) {
 			picked = append(picked, domain)
 		}
 	}
@@ -312,7 +446,8 @@ func whichZonesToProcess(domains []*models.DomainConfig, filter string) []*model
 func splitConcurrent(domains []*models.DomainConfig, filter string) (serial []*models.DomainConfig, concurrent []*models.DomainConfig) {
 	if filter == "none" {
 		return domains, nil
-	} else if filter == "all" {
+	}
+	if filter == "all" {
 		return nil, domains
 	}
 	for _, dc := range domains {
@@ -329,12 +464,12 @@ func splitConcurrent(domains []*models.DomainConfig, filter string) (serial []*m
 // concurrency.  Otherwise false is returned.
 func allConcur(dc *models.DomainConfig) bool {
 	if !providers.ProviderHasCapability(dc.RegistrarInstance.ProviderType, providers.CanConcur) {
-		//fmt.Printf("WHY? %q: %+v\n", dc.Name, dc.RegistrarInstance)
+		// fmt.Printf("WHY? %q: %+v\n", dc.Name, dc.RegistrarInstance)
 		return false
 	}
 	for _, p := range dc.DNSProviderInstances {
 		if !providers.ProviderHasCapability(p.ProviderType, providers.CanConcur) {
-			//fmt.Printf("WHY? %q: %+v\n", dc.Name, p)
+			// fmt.Printf("WHY? %q: %+v\n", dc.Name, p)
 			return false
 		}
 	}
@@ -361,34 +496,47 @@ func optimizeOrder(zones []*models.DomainConfig) []*models.DomainConfig {
 	return zones
 }
 
-func oneZone(zone *models.DomainConfig, args PPreviewArgs, zc *zoneCache) {
+func oneZonePopulate(zone *models.DomainConfig, zc *cmdZoneCache) error {
+	var errs []error
+	// Loop over all the providers configured for that zone:
+	for _, provider := range zone.DNSProviderInstances {
+		populateCorrections, err := generatePopulateCorrections(provider, zone, zc)
+		if err != nil {
+			errs = append(errs, err)
+		}
+		zone.StorePopulateCorrections(provider.Name, populateCorrections)
+	}
+	return errors.Join(errs...)
+}
+
+func oneZone(zone *models.DomainConfig, args PPreviewArgs) error {
+	var errs []error
 	// Fix the parent zone's delegation: (if able/needed)
-	//zone.NameserversMutex.Lock()
-	delegationCorrections := generateDelegationCorrections(zone, zone.DNSProviderInstances, zone.RegistrarInstance)
-	//zone.NameserversMutex.Unlock()
+	delegationCorrections, dcCount, err := generateDelegationCorrections(zone, zone.DNSProviderInstances, zone.RegistrarInstance)
+	if err != nil {
+		errs = append(errs, err)
+	}
 
 	// Loop over the (selected) providers configured for that zone:
 	providersToProcess := whichProvidersToProcess(zone.DNSProviderInstances, args.Providers)
 	for _, provider := range providersToProcess {
-
-		// Populate the zones at the provider (if desired/needed/able):
-		if !args.NoPopulate {
-			populateCorrections := generatePopulateCorrections(provider, zone.Name, zc)
-			zone.StoreCorrections(provider.Name, populateCorrections)
-		}
-
 		// Update the zone's records at the provider:
-		zoneCor, rep := generateZoneCorrections(zone, provider)
+		zoneCor, rep, actualChangeCount, err := generateZoneCorrections(zone, provider)
 		zone.StoreCorrections(provider.Name, rep)
 		zone.StoreCorrections(provider.Name, zoneCor)
+		zone.IncrementChangeCount(provider.Name, actualChangeCount)
+		if err != nil {
+			errs = append(errs, err)
+		}
 	}
 
 	// Do the delegation corrections after the zones are updated.
 	zone.StoreCorrections(zone.RegistrarInstance.Name, delegationCorrections)
+	zone.IncrementChangeCount(zone.RegistrarInstance.Name, dcCount)
+	return errors.Join(errs...)
 }
 
 func whichProvidersToProcess(providers []*models.DNSProviderInstance, filter string) []*models.DNSProviderInstance {
-
 	if filter == "all" { // all
 		return providers
 	}
@@ -423,20 +571,51 @@ func skipProvider(name string, providers []*models.DNSProviderInstance) bool {
 	})
 }
 
-func genReportItem(zname string, corrections []*models.Correction, pname string) *ReportItem {
+func parseCorrectionMsg(s string) []string {
+	// Regex to remove the terminal styled formatting
+	ansiRe := regexp.MustCompile(`\x1b\[[0-9;]*[a-zA-Z]`)
+	s = ansiRe.ReplaceAllString(s, "")
+	// Create a slice(array) of correction/actions/changes from Msg
+	corrections := strings.Split(s, "\n")
+	// Clean up the slice, precaution remove any empty entries.
+	clean := make([]string, 0, len(corrections))
+	for _, l := range corrections {
+		l = strings.TrimSpace(l)
+		if l != "" {
+			clean = append(clean, l)
+		}
+	}
+	return clean
+}
 
-	// Only count the actions, not the messages.
-	cnt := 0
+func genReportItem(zoneName string, corrections []*models.Correction, providerName string, registrarName string) *ReportItem {
+	correctionDetails := make([]string, 0)
 	for _, cor := range corrections {
 		if cor.F != nil {
-			cnt++
+			// `corrections` is a list that contains "informational" messages
+			// (where `.F = nil`) and "actions" to be taken (where `.F != nil`).
+			// When `.F = nil`, the contents of `.Msg` can either be a concatenation of all
+			// actions(all changes done in a single API call) or a single
+			// action(one API call per change), depending on the provider's implementation.
+			//
+			// We are parsing `cor.Msg` to remove terminal styled formatting and create
+			// a comprehensive list of actions (changes), as well as get an accurate
+			// number of corrections (`len(correctionDetails)`).
+
+			correctionDetails = append(correctionDetails, parseCorrectionMsg(cor.Msg)...)
 		}
 	}
 
 	r := ReportItem{
-		Domain:      zname,
-		Corrections: cnt,
-		Provider:    pname,
+		Domain:            zoneName,
+		Corrections:       len(correctionDetails),
+		CorrectionDetails: correctionDetails,
+	}
+	if providerName != "" {
+		r.Provider = providerName
+	}
+	if registrarName != "" {
+		r.Registrar = registrarName
 	}
 	return &r
 }
@@ -449,7 +628,6 @@ func pprintOrRunCorrections(zoneName string, providerName string, corrections []
 	cc := 0
 	cn := 0
 	for _, correction := range corrections {
-
 		// Print what we're about to do.
 		if correction.F == nil {
 			out.PrintReport(cn, correction)
@@ -459,23 +637,25 @@ func pprintOrRunCorrections(zoneName string, providerName string, corrections []
 			cc++
 		}
 
-		var err error
-		if push {
+		// If interactive, ask "are you sure?" and skip if not.
+		if push && interactive && !out.PromptToRun() {
+			continue
+		}
 
-			// If interactive, ask "are you sure?" and skip if not.
-			if interactive && !out.PromptToRun() {
-				continue
-			}
-
-			// If it is an action (not an informational message), notify and execute.
-			if correction.F != nil {
-				notifier.Notify(zoneName, providerName, correction.Msg, err, false)
+		// If it is an action (not an informational message), notify and execute.
+		if correction.F != nil {
+			var err error
+			if push {
 				err = correction.F()
 				out.EndCorrection(err)
 				if err != nil {
 					anyErrors = true
 				}
 			}
+			notifyErr := notifier.Notify(zoneName, providerName, correction.Msg, err, !push)
+			if notifyErr != nil {
+				out.Warnf("Error sending notification: %s\n", notifyErr)
+			}
 		}
 	}
 
@@ -489,76 +669,82 @@ func writeReport(report string, reportItems []*ReportItem) error {
 		return nil
 	}
 
-	f, err := os.OpenFile(report, os.O_CREATE|os.O_WRONLY|os.O_TRUNC, 0644)
+	f, err := os.OpenFile(report, os.O_CREATE|os.O_WRONLY|os.O_TRUNC, 0o644)
 	if err != nil {
 		return err
 	}
 	defer f.Close()
-	b, err := json.MarshalIndent(reportItems, "", "  ")
-	if err != nil {
-		return err
-	}
-	if _, err := f.Write(b); err != nil {
+
+	// Disabling HTML encoding
+	enc := json.NewEncoder(f)
+	enc.SetIndent("", "  ")
+	enc.SetEscapeHTML(false)
+
+	if err := enc.Encode(reportItems); err != nil {
 		return err
 	}
 	return nil
 }
 
-func generatePopulateCorrections(provider *models.DNSProviderInstance, zoneName string, zcache *zoneCache) []*models.Correction {
-
+func generatePopulateCorrections(provider *models.DNSProviderInstance, zone *models.DomainConfig, zcache *cmdZoneCache) ([]*models.Correction, error) {
 	lister, ok := provider.Driver.(providers.ZoneLister)
 	if !ok {
-		return nil // We can't generate a list. No corrections are possible.
+		return nil, nil // We can't generate a list. No corrections are possible.
 	}
 
 	z, err := zcache.zoneList(provider.Name, lister)
 	if err != nil {
-		return []*models.Correction{{Msg: fmt.Sprintf("zoneList failed for %q: %s", provider.Name, err)}}
+		errMsg := fmt.Sprintf("zoneList failed for %q: %s", provider.Name, err)
+		return []*models.Correction{{Msg: errMsg}}, errors.New(errMsg)
 	}
 	zones := *z
 
-	aceZoneName, _ := idna.ToASCII(zoneName)
+	aceZoneName, _ := idna.ToASCII(zone.Name)
 	if slices.Contains(zones, aceZoneName) {
-		return nil // zone exists. Nothing to do.
+		return nil, nil // zone exists. Nothing to do.
 	}
 
 	creator, ok := provider.Driver.(providers.ZoneCreator)
 	if !ok {
-		return []*models.Correction{{Msg: fmt.Sprintf("Zone %q does not exist. Can not create because %q does not implement ZoneCreator", aceZoneName, provider.Name)}}
+		errMsg := fmt.Sprintf("Zone %q does not exist. Can not create because %q does not implement ZoneCreator", aceZoneName, provider.Name)
+		return []*models.Correction{{Msg: errMsg}}, errors.New(errMsg)
 	}
 
 	return []*models.Correction{{
-		Msg: fmt.Sprintf("Create zone '%s' in the '%s' profile", aceZoneName, provider.Name),
-		F:   func() error { return creator.EnsureZoneExists(aceZoneName) },
-	}}
+		Msg: fmt.Sprintf("Ensuring zone %q exists in %q", aceZoneName, provider.Name),
+		F:   func() error { return creator.EnsureZoneExists(aceZoneName, zone.Metadata) },
+	}}, nil
 }
 
-func generateZoneCorrections(zone *models.DomainConfig, provider *models.DNSProviderInstance) ([]*models.Correction, []*models.Correction) {
-	reports, zoneCorrections, err := zonerecs.CorrectZoneRecords(provider.Driver, zone)
+func generateZoneCorrections(zone *models.DomainConfig, provider *models.DNSProviderInstance) ([]*models.Correction, []*models.Correction, int, error) {
+	reports, zoneCorrections, actualChangeCount, err := zonerecs.CorrectZoneRecords(provider.Driver, zone)
 	if err != nil {
-		return []*models.Correction{{Msg: fmt.Sprintf("Domain %q provider %s Error: %s", zone.Name, provider.Name, err)}}, nil
+		return []*models.Correction{{Msg: fmt.Sprintf("Domain %q provider %s Error: %s", zone.Name, provider.Name, err)}}, nil, 0, err
 	}
-	return zoneCorrections, reports
+	return zoneCorrections, reports, actualChangeCount, nil
 }
 
-func generateDelegationCorrections(zone *models.DomainConfig, providers []*models.DNSProviderInstance, _ *models.RegistrarInstance) []*models.Correction {
-	//fmt.Printf("DEBUG: generateDelegationCorrections start zone=%q nsList = %v\n", zone.Name, zone.Nameservers)
+func generateDelegationCorrections(zone *models.DomainConfig, providers []*models.DNSProviderInstance, _ *models.RegistrarInstance) ([]*models.Correction, int, error) {
+	// fmt.Printf("DEBUG: generateDelegationCorrections start zone=%q nsList = %v\n", zone.Name, zone.Nameservers)
 	nsList, err := nameservers.DetermineNameserversForProviders(zone, providers, true)
 	if err != nil {
-		return msg(fmt.Sprintf("DtermineNS: zone %q; Error: %s", zone.Name, err))
+		return msg(fmt.Sprintf("DetermineNS: zone %q; Error: %s", zone.Name, err)), 0, err
 	}
 	zone.Nameservers = nsList
 	nameservers.AddNSRecords(zone)
 
 	if len(zone.Nameservers) == 0 && zone.Metadata["no_ns"] != "true" {
-		return []*models.Correction{{Msg: fmt.Sprintf("No nameservers declared for domain %q; skipping registrar. Add {no_ns:'true'} to force", zone.Name)}}
+		return []*models.Correction{{Msg: fmt.Sprintf("Skipping registrar %q: No nameservers declared for domain %q. Add {no_ns:'true'} to force",
+			zone.RegistrarName,
+			zone.Name,
+		)}}, 0, nil
 	}
 
 	corrections, err := zone.RegistrarInstance.Driver.GetRegistrarCorrections(zone)
 	if err != nil {
-		return msg(fmt.Sprintf("zone %q; Rprovider %q; Error: %s", zone.Name, zone.RegistrarInstance.Name, err))
+		return msg(fmt.Sprintf("zone %q; Rprovider %q; Error: %s", zone.Name, zone.RegistrarInstance.Name, err)), 0, err
 	}
-	return corrections
+	return corrections, len(corrections), nil
 }
 
 func msg(s string) []*models.Correction {
@@ -589,7 +775,7 @@ func PInitializeProviders(cfg *models.DNSConfig, providerConfigs map[string]map[
 		fmt.Fprintln(os.Stderr, strings.Join(msgs, "\n"))
 	}
 	if err != nil {
-		return
+		return notify, err
 	}
 
 	registrars := map[string]providers.Registrar{}
@@ -618,7 +804,7 @@ func PInitializeProviders(cfg *models.DNSConfig, providerConfigs map[string]map[
 			pInst.IsDefault = !isNonDefault[pInst.Name]
 		}
 	}
-	return
+	return notify, err
 }
 
 // pproviderTypeFieldName is the name of the field in creds.json that specifies the provider type id.
@@ -709,7 +895,6 @@ func puniqueStrings(stringSlice []string) []string {
 }
 
 func prefineProviderType(credEntryName string, t string, credFields map[string]string, source string) (replacementType string, warnMsg string, err error) {
-
 	// t="" and t="-" are processed the same. Standardize on "-" to reduce the number of cases to check.
 	if t == "" {
 		t = "-"
@@ -722,7 +907,7 @@ func prefineProviderType(credEntryName string, t string, credFields map[string]s
 	// - or ""  GANDI        lookup worked. Nothing to say.
 	// - or ""  - or ""      ERROR "creds.json has invalid or missing data"
 	// GANDI    ""           WARNING "Working but.... Please fix as follows..."
-	// GANDI    GANDI        INFO "working but unneeded: clean up as follows..."
+	// GANDI    INFO "working but unneeded: clean up as follows..."
 	// GANDI    NAMEDOT      ERROR "error mismatched: please fix as follows..."
 
 	// ERROR: Invalid.
@@ -813,5 +998,4 @@ func prefineProviderType(credEntryName string, t string, credFields map[string]s
 		// use the value in creds.json (this should be the normal case)
 		return ct, "", nil
 	}
-
 }
