@@ -4,7 +4,7 @@ import (
 	"testing"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
-	_ "github.com/StackExchange/dnscontrol/v4/providers/_all"
+	_ "github.com/StackExchange/dnscontrol/v4/pkg/providers/_all"
 )
 
 func TestR53Test_1(t *testing.T) {
