@@ -62,10 +62,14 @@ func helperRRtoRC(rr dns.RR, origin string, fixBug bool) (RecordConfig, error) {
 		err = rc.SetTargetMX(v.Preference, v.Mx)
 	case *dns.NAPTR:
 		err = rc.SetTargetNAPTR(v.Order, v.Preference, v.Flags, v.Service, v.Regexp, v.Replacement)
+	case *dns.OPENPGPKEY:
+		err = rc.SetTarget(v.PublicKey)
 	case *dns.NS:
 		err = rc.SetTarget(v.Ns)
 	case *dns.PTR:
 		err = rc.SetTarget(v.Ptr)
+	case *dns.SMIMEA:
+		err = rc.SetTargetSMIMEA(v.Usage, v.Selector, v.MatchingType, v.Certificate)
 	case *dns.SOA:
 		err = rc.SetTargetSOA(v.Ns, v.Mbox, v.Serial, v.Refresh, v.Retry, v.Expire, v.Minttl)
 	case *dns.SRV:
