@@ -90,7 +90,7 @@ func TestRecordConfig_Copy(t *testing.T) {
 		TlsaMatchingType uint8
 		R53Alias         map[string]string
 		AzureAlias       map[string]string
-		Original         interface{}
+		Original         any
 	}
 	tests := []struct {
 		name    string
@@ -136,7 +136,7 @@ func TestRecordConfig_Copy(t *testing.T) {
 				TlsaMatchingType: 3,
 				R53Alias:         map[string]string{"a": "eh", "b": "bee"},
 				AzureAlias:       map[string]string{"az": "az", "ure": "your"},
-				// Original         interface{},
+				// Original         any,
 			},
 			want: &RecordConfig{
 				Type:             "type",
@@ -174,7 +174,7 @@ func TestRecordConfig_Copy(t *testing.T) {
 				TlsaMatchingType: 3,
 				R53Alias:         map[string]string{"a": "eh", "b": "bee"},
 				AzureAlias:       map[string]string{"az": "az", "ure": "your"},
-				// Original         interface{},
+				// Original         any,
 			},
 		},
 	}
@@ -248,7 +248,7 @@ func TestFixPosition(t *testing.T) {
 		{
 			name: "random string",
 			pos:  "alsdjfsljd",
-			want: "[line:alsdjfsljd]",
+			want: "[alsdjfsljd]",
 		},
 	}
 	for _, tt := range tests {
