@@ -60,48 +60,55 @@ func makeRec(label, rtype, content string) *models.RecordConfig {
 	origin := "f.com"
 	r := models.RecordConfig{TTL: 300}
 	r.SetLabel(label, origin)
-	r.PopulateFromString(rtype, content, origin)
+	if err := r.PopulateFromString(rtype, content, origin); err != nil {
+		panic(err)
+	}
 	return &r
 }
+
 func makeRecTTL(label, rtype, content string, ttl uint32) *models.RecordConfig {
 	r := makeRec(label, rtype, content)
 	r.TTL = ttl
 	return r
 }
 
-var testDataAA1234 = makeRec("laba", "A", "1.2.3.4")               // [ 0]
-var testDataAA5678 = makeRec("laba", "A", "5.6.7.8")               //
-var testDataAA1234ttl700 = makeRecTTL("laba", "A", "1.2.3.4", 700) //
-var testDataAA5678ttl700 = makeRecTTL("laba", "A", "5.6.7.8", 700) //
-var testDataAMX10a = makeRec("laba", "MX", "10 laba")              // [ 1]
-var testDataCCa = makeRec("labc", "CNAME", "laba")                 // [ 2]
-var testDataEA15 = makeRec("labe", "A", "10.10.10.15")             // [ 3]
-var e4 = makeRec("labe", "A", "10.10.10.16")                       // [ 4]
-var e5 = makeRec("labe", "A", "10.10.10.17")                       // [ 5]
-var e6 = makeRec("labe", "A", "10.10.10.18")                       // [ 6]
-var e7 = makeRec("labg", "NS", "laba")                             // [ 7]
-var e8 = makeRec("labg", "NS", "labb")                             // [ 8]
-var e9 = makeRec("labg", "NS", "labc")                             // [ 9]
-var e10 = makeRec("labg", "NS", "labe")                            // [10]
-var e11mx = makeRec("labh", "MX", "22 ttt")                        // [11]
-var e11 = makeRec("labh", "CNAME", "labd")                         // [11]
-var testDataApexMX1aaa = makeRec("", "MX", "1 aaa")
+var (
+	testDataAA1234       = makeRec("laba", "A", "1.2.3.4")         // [ 0]
+	testDataAA5678       = makeRec("laba", "A", "5.6.7.8")         //
+	testDataAA1234ttl700 = makeRecTTL("laba", "A", "1.2.3.4", 700) //
+	testDataAA5678ttl700 = makeRecTTL("laba", "A", "5.6.7.8", 700) //
+	testDataAMX10a       = makeRec("laba", "MX", "10 laba")        // [ 1]
+	testDataCCa          = makeRec("labc", "CNAME", "laba")        // [ 2]
+	testDataEA15         = makeRec("labe", "A", "10.10.10.15")     // [ 3]
+	e4                   = makeRec("labe", "A", "10.10.10.16")     // [ 4]
+	e5                   = makeRec("labe", "A", "10.10.10.17")     // [ 5]
+	e6                   = makeRec("labe", "A", "10.10.10.18")     // [ 6]
+	e7                   = makeRec("labg", "NS", "laba")           // [ 7]
+	e8                   = makeRec("labg", "NS", "labb")           // [ 8]
+	e9                   = makeRec("labg", "NS", "labc")           // [ 9]
+	e10                  = makeRec("labg", "NS", "labe")           // [10]
+	e11mx                = makeRec("labh", "MX", "22 ttt")         // [11]
+	e11                  = makeRec("labh", "CNAME", "labd")        // [11]
+	testDataApexMX1aaa   = makeRec("", "MX", "1 aaa")
+)
 
-var testDataAA1234clone = makeRec("laba", "A", "1.2.3.4") // [ 0']
-var testDataAA12345 = makeRec("laba", "A", "1.2.3.5")     // [ 1']
-var testDataAMX20b = makeRec("laba", "MX", "20 labb")     // [ 2']
-var d3 = makeRec("labe", "A", "10.10.10.95")              // [ 3']
-var d4 = makeRec("labe", "A", "10.10.10.96")              // [ 4']
-var d5 = makeRec("labe", "A", "10.10.10.97")              // [ 5']
-var d6 = makeRec("labe", "A", "10.10.10.98")              // [ 6']
-var d7 = makeRec("labf", "TXT", "foo")                    // [ 7']
-var d8 = makeRec("labg", "NS", "labf")                    // [ 8']
-var d9 = makeRec("labg", "NS", "laba")                    // [ 9']
-var d10 = makeRec("labg", "NS", "labe")                   // [10']
-var d11 = makeRec("labg", "NS", "labb")                   // [11']
-var d12 = makeRec("labh", "A", "1.2.3.4")                 // [12']
-var d13 = makeRec("labc", "CNAME", "labe")                // [13']
-var testDataApexMX22bbb = makeRec("", "MX", "22 bbb")
+var (
+	testDataAA1234clone = makeRec("laba", "A", "1.2.3.4")     // [ 0']
+	testDataAA12345     = makeRec("laba", "A", "1.2.3.5")     // [ 1']
+	testDataAMX20b      = makeRec("laba", "MX", "20 labb")    // [ 2']
+	d3                  = makeRec("labe", "A", "10.10.10.95") // [ 3']
+	d4                  = makeRec("labe", "A", "10.10.10.96") // [ 4']
+	d5                  = makeRec("labe", "A", "10.10.10.97") // [ 5']
+	d6                  = makeRec("labe", "A", "10.10.10.98") // [ 6']
+	d7                  = makeRec("labf", "TXT", "foo")       // [ 7']
+	d8                  = makeRec("labg", "NS", "labf")       // [ 8']
+	d9                  = makeRec("labg", "NS", "laba")       // [ 9']
+	d10                 = makeRec("labg", "NS", "labe")       // [10']
+	d11                 = makeRec("labg", "NS", "labb")       // [11']
+	d12                 = makeRec("labh", "A", "1.2.3.4")     // [12']
+	d13                 = makeRec("labc", "CNAME", "labe")    // [13']
+	testDataApexMX22bbb = makeRec("", "MX", "22 bbb")
+)
 
 func justMsgString(cl ChangeList) string {
 	msgs := justMsgs(cl)
@@ -127,7 +134,13 @@ func compareCL(t *testing.T, fnname, testname, testpart string, gotcl ChangeList
 	if d != "" {
 		t.Errorf("%s()/%s (wantChange%s):\n===got===\n%s\n===want===\n%s\n===diff===\n%s\n===", fnname, testname, testpart, gs, ws, d)
 	}
+}
 
+func compareACC(t *testing.T, fnname, testname, testpart string, gotacc int, wantacc int) {
+	t.Helper()
+	if gotacc != wantacc {
+		t.Errorf("%s()/%s (wantChange%s):\n===got===\n%v\n===want===\n%v\n", fnname, testname, testpart, gotacc, wantacc)
+	}
 }
 
 func Test_analyzeByRecordSet(t *testing.T) {
@@ -150,8 +163,8 @@ func Test_analyzeByRecordSet(t *testing.T) {
 		wantChangeRec   string
 		wantMsgsRec     string
 		wantChangeZone  string
+		wantChangeCount int
 	}{
-
 		{
 			name: "oneequal",
 			args: args{
@@ -159,6 +172,7 @@ func Test_analyzeByRecordSet(t *testing.T) {
 				existing: models.Records{testDataAA1234},
 				desired:  models.Records{testDataAA1234clone},
 			},
+			wantChangeCount: 0,
 			wantMsgs:        "", // Empty
 			wantChangeRSet:  "ChangeList: len=0",
 			wantChangeLabel: "ChangeList: len=0",
@@ -172,7 +186,8 @@ func Test_analyzeByRecordSet(t *testing.T) {
 				existing: models.Records{testDataAA1234, testDataAMX10a},
 				desired:  models.Records{testDataAA1234clone, testDataAMX20b},
 			},
-			wantMsgs: "± MODIFY laba.f.com MX (10 laba.f.com. ttl=300) -> (20 labb.f.com. ttl=300)",
+			wantChangeCount: 1,
+			wantMsgs:        "± MODIFY laba.f.com MX (10 laba.f.com. ttl=300) -> (20 labb.f.com. ttl=300)",
 			wantChangeRSet: `
 ChangeList: len=1
 00: Change: verb=CHANGE
@@ -206,7 +221,8 @@ ChangeList: len=1
 				existing: models.Records{testDataAA1234, testDataApexMX1aaa},
 				desired:  models.Records{testDataAA1234clone, testDataApexMX22bbb},
 			},
-			wantMsgs: "± MODIFY f.com MX (1 aaa.f.com. ttl=300) -> (22 bbb.f.com. ttl=300)",
+			wantChangeCount: 1,
+			wantMsgs:        "± MODIFY f.com MX (1 aaa.f.com. ttl=300) -> (22 bbb.f.com. ttl=300)",
 			wantChangeRSet: `
 ChangeList: len=1
 00: Change: verb=CHANGE
@@ -240,6 +256,7 @@ ChangeList: len=1
 				existing: models.Records{testDataAA1234, testDataAMX10a},
 				desired:  models.Records{testDataAA1234clone, testDataAA12345, testDataAMX20b},
 			},
+			wantChangeCount: 2,
 			wantMsgs: `
 ± MODIFY laba.f.com MX (10 laba.f.com. ttl=300) -> (20 labb.f.com. ttl=300)
 + CREATE laba.f.com A 1.2.3.5 ttl=300
@@ -290,6 +307,7 @@ ChangeList: len=2
 				existing: models.Records{testDataAA1234, testDataCCa},
 				desired:  models.Records{d13, d3},
 			},
+			wantChangeCount: 3,
 			wantMsgs: `
 + CREATE labe.f.com A 10.10.10.95 ttl=300
 ± MODIFY labc.f.com CNAME (laba.f.com. ttl=300) -> (labe.f.com. ttl=300)
@@ -352,6 +370,7 @@ ChangeList: len=3
 				existing: models.Records{testDataAA1234, testDataAMX10a, testDataCCa, testDataEA15, e4, e5, e6, e7, e8, e9, e10, e11},
 				desired:  models.Records{testDataAA1234clone, testDataAA12345, testDataAMX20b, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12},
 			},
+			wantChangeCount: 11,
 			wantMsgs: `
 + CREATE labf.f.com TXT "foo" ttl=300
 ± MODIFY labg.f.com NS (labc.f.com. ttl=300) -> (labf.f.com. ttl=300)
@@ -525,25 +544,27 @@ ChangeList: len=11
 		// Therefore we have to run NewCompareConfig() each time.
 
 		t.Run(tt.name, func(t *testing.T) {
-			cl := analyzeByRecordSet(NewCompareConfig(tt.args.origin, tt.args.existing, tt.args.desired, tt.args.compFn))
+			cl, actualChangeCount := analyzeByRecordSet(NewCompareConfig(tt.args.origin, tt.args.existing, tt.args.desired, tt.args.compFn))
 			compareMsgs(t, "analyzeByRecordSet", tt.name, "RSet", cl, tt.wantMsgsRSet, tt.wantMsgs)
 			compareCL(t, "analyzeByRecordSet", tt.name, "RSet", cl, tt.wantChangeRSet)
+			compareACC(t, "analyzeByRecordSet", tt.name, "ACC", actualChangeCount, tt.wantChangeCount)
 		})
 
 		t.Run(tt.name, func(t *testing.T) {
-			cl := analyzeByLabel(NewCompareConfig(tt.args.origin, tt.args.existing, tt.args.desired, tt.args.compFn))
+			cl, actualChangeCount := analyzeByLabel(NewCompareConfig(tt.args.origin, tt.args.existing, tt.args.desired, tt.args.compFn))
 			compareMsgs(t, "analyzeByLabel", tt.name, "Label", cl, tt.wantMsgsLabel, tt.wantMsgs)
 			compareCL(t, "analyzeByLabel", tt.name, "Label", cl, tt.wantChangeLabel)
+			compareACC(t, "analyzeByLabel", tt.name, "ACC", actualChangeCount, tt.wantChangeCount)
 		})
 
 		t.Run(tt.name, func(t *testing.T) {
-			cl := analyzeByRecord(NewCompareConfig(tt.args.origin, tt.args.existing, tt.args.desired, tt.args.compFn))
+			cl, actualChangeCount := analyzeByRecord(NewCompareConfig(tt.args.origin, tt.args.existing, tt.args.desired, tt.args.compFn))
 			compareMsgs(t, "analyzeByRecord", tt.name, "Rec", cl, tt.wantMsgsRec, tt.wantMsgs)
 			compareCL(t, "analyzeByRecord", tt.name, "Rec", cl, tt.wantChangeRec)
+			compareACC(t, "analyzeByRecord", tt.name, "ACC", actualChangeCount, tt.wantChangeCount)
 		})
 
 		// NB(tlim): There is no analyzeByZone().  diff2.ByZone() uses analyzeByRecord().
-
 	}
 }
 
@@ -570,7 +591,7 @@ func mkTargetConfig(x ...*models.RecordConfig) []targetConfig {
 }
 
 func mkTargetConfigMap(x ...*models.RecordConfig) map[string]*targetConfig {
-	var m = map[string]*targetConfig{}
+	m := map[string]*targetConfig{}
 	for _, v := range mkTargetConfig(x...) {
 		m[v.comparableFull] = &v
 	}
@@ -578,7 +599,6 @@ func mkTargetConfigMap(x ...*models.RecordConfig) map[string]*targetConfig {
 }
 
 func Test_diffTargets(t *testing.T) {
-
 	type args struct {
 		existing []targetConfig
 		desired  []targetConfig
@@ -588,7 +608,6 @@ func Test_diffTargets(t *testing.T) {
 		args args
 		want ChangeList
 	}{
-
 		{
 			name: "add1changettl",
 			args: args{
@@ -596,11 +615,12 @@ func Test_diffTargets(t *testing.T) {
 				desired:  mkTargetConfig(testDataAA5678ttl700, testDataAA1234ttl700),
 			},
 			want: ChangeList{
-				Change{Type: CHANGE,
-					Key: models.RecordKey{NameFQDN: "laba.f.com", Type: "A"},
-					New: models.Records{testDataAA5678ttl700, testDataAA1234ttl700},
+				Change{
+					Type: CHANGE,
+					Key:  models.RecordKey{NameFQDN: "laba.f.com", Type: "A"},
+					New:  models.Records{testDataAA5678ttl700, testDataAA1234ttl700},
 					Msgs: []string{
-						"± MODIFY-TTL laba.f.com A 5.6.7.8 ttl=(300->700)",
+						"± MODIFY-TTL laba.f.com A ttl=(300->700) 5.6.7.8",
 						"+ CREATE laba.f.com A 1.2.3.4 ttl=700",
 					},
 				},
@@ -613,7 +633,7 @@ func Test_diffTargets(t *testing.T) {
 				existing: mkTargetConfig(testDataAA1234),
 				desired:  mkTargetConfig(testDataAA1234),
 			},
-			//want: ,
+			// want: ,
 		},
 
 		{
@@ -623,7 +643,8 @@ func Test_diffTargets(t *testing.T) {
 				desired:  mkTargetConfig(testDataAA1234, testDataAMX10a),
 			},
 			want: ChangeList{
-				Change{Type: CREATE,
+				Change{
+					Type: CREATE,
 					Key:  models.RecordKey{NameFQDN: "laba.f.com", Type: "MX"},
 					New:  models.Records{makeRec("laba", "MX", "10 laba.f.com.")},
 					Msgs: []string{"+ CREATE laba.f.com MX 10 laba.f.com. ttl=300"},
@@ -638,7 +659,8 @@ func Test_diffTargets(t *testing.T) {
 				desired:  mkTargetConfig(testDataAA1234),
 			},
 			want: ChangeList{
-				Change{Type: DELETE,
+				Change{
+					Type: DELETE,
 					Key:  models.RecordKey{NameFQDN: "laba.f.com", Type: "MX"},
 					Old:  models.Records{makeRec("laba", "MX", "10 laba.f.com.")},
 					Msgs: []string{"- DELETE laba.f.com MX 10 laba.f.com. ttl=300"},
@@ -653,7 +675,8 @@ func Test_diffTargets(t *testing.T) {
 				desired:  mkTargetConfig(testDataAA1234, testDataAMX20b),
 			},
 			want: ChangeList{
-				Change{Type: CHANGE,
+				Change{
+					Type: CHANGE,
 					Key:  models.RecordKey{NameFQDN: "laba.f.com", Type: "MX"},
 					Old:  models.Records{testDataAMX10a},
 					New:  models.Records{testDataAMX20b},
@@ -669,7 +692,8 @@ func Test_diffTargets(t *testing.T) {
 				desired:  mkTargetConfig(testDataAA1234),
 			},
 			want: ChangeList{
-				Change{Type: CHANGE,
+				Change{
+					Type: CHANGE,
 					Key:  models.RecordKey{NameFQDN: "laba.f.com", Type: "A"},
 					Old:  models.Records{testDataAA1234, testDataAA5678},
 					New:  models.Records{testDataAA1234},
@@ -680,13 +704,13 @@ func Test_diffTargets(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			//fmt.Printf("DEBUG: Test %02d\n", i)
+			// fmt.Printf("DEBUG: Test %02d\n", i)
 			got := diffTargets(tt.args.existing, tt.args.desired)
 			g := strings.TrimSpace(justMsgString(got))
 			w := strings.TrimSpace(justMsgString(tt.want))
 			d := diff.Diff(g, w)
 			if d != "" {
-				//fmt.Printf("DEBUG: fail %q %q\n", g, w)
+				// fmt.Printf("DEBUG: fail %q %q\n", g, w)
 				t.Errorf("diffTargets()\n diff=%s", d)
 			}
 		})
@@ -704,7 +728,6 @@ func Test_removeCommon(t *testing.T) {
 		want  []targetConfig
 		want1 []targetConfig
 	}{
-
 		{
 			name: "same",
 			args: args{
@@ -756,7 +779,6 @@ func Test_filterBy(t *testing.T) {
 		args args
 		want []targetConfig
 	}{
-
 		{
 			name: "removeall",
 			args: args{
@@ -805,7 +827,6 @@ func Test_splitTTLOnly(t *testing.T) {
 		wantDesireDiff []targetConfig
 		wantChanges    string
 	}{
-
 		{
 			name: "simple",
 			args: args{
@@ -814,7 +835,7 @@ func Test_splitTTLOnly(t *testing.T) {
 			},
 			wantExistDiff:  nil,
 			wantDesireDiff: nil,
-			wantChanges:    "ChangeList: len=1\n00: Change: verb=CHANGE\n    key={laba.f.com A}\n    Hints=OnlyTTL\n{laba.f.com A}    old=[1.2.3.4]\n    new=[1.2.3.4]\n    msg=[\"± MODIFY-TTL laba.f.com A 1.2.3.4 ttl=(300->700)\"]\n",
+			wantChanges:    "ChangeList: len=1\n00: Change: verb=CHANGE\n    key={laba.f.com A}\n    Hints=OnlyTTL\n{laba.f.com A}    old=[1.2.3.4]\n    new=[1.2.3.4]\n    msg=[\"± MODIFY-TTL laba.f.com A ttl=(300->700) 1.2.3.4\"]\n",
 		},
 
 		{
