@@ -43,7 +43,7 @@ func Test_domaintree(t *testing.T) {
 func executeTreeTest(inputs []string, founds []string, missings []string) func(*testing.T) {
 	return func(t *testing.T) {
 		t.Helper()
-		tree := dnstree.Create[interface{}]()
+		tree := dnstree.Create[any]()
 		for _, input := range inputs {
 			tree.Set(input, struct{}{})
 		}
