@@ -331,6 +331,28 @@ var AAAA = recordBuilder('AAAA');
 // AKAMAICDN(name, target, recordModifiers...)
 var AKAMAICDN = recordBuilder('AKAMAICDN');
 
+// AKAMAITLC(name, answer_type, target, recordModifiers...)
+var AKAMAITLC = recordBuilder('AKAMAITLC', {
+    args: [
+        ['name', _.isString],
+        [
+            'answer_type',
+            function (value) {
+                return (
+                    _.isString(value) &&
+                    ['DUAL', 'A', 'AAAA'].indexOf(value) !== -1
+                );
+            },
+        ],
+        ['target', _.isString],
+    ],
+    transform: function (record, args, modifier) {
+        record.name = args.name;
+        record.answer_type = args.answer_type;
+        record.target = args.target;
+    },
+});
+
 // ALIAS(name,target, recordModifiers...)
 var ALIAS = recordBuilder('ALIAS');
 
@@ -669,20 +691,20 @@ var TXT = recordBuilder('TXT', {
 });
 
 var LUA = recordBuilder('LUA', {
-  args: [
-    ['name', _.isString],
-    ['rtype', _.isString],
-    ['target', isStringOrArray],
-  ],
-  transform: function (record, args, modifiers) {
-    record.name = args.name;
-    record.luartype = args.rtype.toUpperCase();
-    if (_.isString(args.target)) {
-      record.target = args.target;
-    } else {
-      record.target = args.target.join('');
-    }
-  },
+    args: [
+        ['name', _.isString],
+        ['rtype', _.isString],
+        ['target', isStringOrArray],
+    ],
+    transform: function (record, args, modifiers) {
+        record.name = args.name;
+        record.luartype = args.rtype.toUpperCase();
+        if (_.isString(args.target)) {
+            record.target = args.target;
+        } else {
+            record.target = args.target.join('');
+        }
+    },
 });
 
 // Parses coordinates of the form 41°24'12.2"N 2°10'26.5"E
@@ -1149,6 +1171,29 @@ function NO_PURGE(d) {
     d.KeepUnknown = true;
 }
 
+// IGNORE_EXTERNAL_DNS(prefix)
+// When enabled, DNSControl will automatically detect TXT records created by
+// Kubernetes external-dns and ignore both the TXT records and the corresponding
+// DNS records they manage. External-dns creates TXT records with content like:
+// "heritage=external-dns,external-dns/owner=<owner-id>,external-dns/resource=<resource>"
+// This allows DNSControl to coexist with external-dns in the same zone.
+//
+// Optional prefix parameter: If your external-dns is configured with a custom
+// --txt-prefix (e.g., "extdns-"), pass it here to detect those records.
+// Without a prefix, it detects the default format ("%{record_type}-" prefixes like "a-", "cname-").
+//
+// Usage:
+//   IGNORE_EXTERNAL_DNS()           // Use default detection (a-, cname-, etc.)
+//   IGNORE_EXTERNAL_DNS("extdns-") // Custom prefix
+function IGNORE_EXTERNAL_DNS(prefix) {
+    return function (d) {
+        d.ignore_external_dns = true;
+        if (prefix) {
+            d.external_dns_prefix = prefix;
+        }
+    };
+}
+
 // ENSURE_ABSENT_REC()
 // Usage: A("foo", "1.2.3.4", ENSURE_ABSENT_REC())
 function ENSURE_ABSENT_REC() {
@@ -1308,8 +1353,6 @@ function recordBuilder(type, opts) {
             if (
                 d.subdomain &&
                 record.type != 'CF_SINGLE_REDIRECT' &&
-                record.type != 'CF_REDIRECT' &&
-                record.type != 'CF_TEMP_REDIRECT' &&
                 record.type != 'CF_WORKER_ROUTE' &&
                 record.type != 'ADGUARDHOME_A_PASSTHROUGH' &&
                 record.type != 'ADGUARDHOME_AAAA_PASSTHROUGH'
@@ -1447,28 +1490,6 @@ function _validateCloudflareRedirect(value) {
     return value.indexOf(',') === -1;
 }
 
-var CF_REDIRECT = recordBuilder('CF_REDIRECT', {
-    args: [
-        ['source', _validateCloudflareRedirect],
-        ['destination', _validateCloudflareRedirect],
-    ],
-    transform: function (record, args, modifiers) {
-        record.name = '@';
-        record.target = args.source + ',' + args.destination;
-    },
-});
-
-var CF_TEMP_REDIRECT = recordBuilder('CF_TEMP_REDIRECT', {
-    args: [
-        ['source', _validateCloudflareRedirect],
-        ['destination', _validateCloudflareRedirect],
-    ],
-    transform: function (record, args, modifiers) {
-        record.name = '@';
-        record.target = args.source + ',' + args.destination;
-    },
-});
-
 var CF_WORKER_ROUTE = recordBuilder('CF_WORKER_ROUTE', {
     args: [
         ['pattern', _validateCloudflareRedirect],
@@ -1490,6 +1511,9 @@ var URL = recordBuilder('URL');
 var URL301 = recordBuilder('URL301');
 var FRAME = recordBuilder('FRAME');
 var CLOUDNS_WR = recordBuilder('CLOUDNS_WR');
+/**
+ * @deprecated Please use URL or URL301 instead
+ */
 var PORKBUN_URLFWD = recordBuilder('PORKBUN_URLFWD');
 var BUNNY_DNS_RDR = recordBuilder('BUNNY_DNS_RDR');
 // LOC_BUILDER_DD takes an object:
@@ -2422,9 +2446,20 @@ function rawrecordBuilder(type) {
             rawArgs.push(arguments[i]);
         }
 
+        // Record which line called this record type.
+        // NB(tlim): Hopefully we can find a better way to do this in the
+        // future. Right now we're faking that there was an error just to parse
+        // out the line number. That's inefficient but I can't find anything better.
+        // This will certainly break if we change to a different Javascript interpreter.
+        // Hopefully any other interpreter will have a better way to do this.
+        var positionLines = new Error().stack.split('\n');
+        var position = positionLines[positionLines.length - 2];
+
         return function (d) {
             var record = {
                 type: type,
+                filepos: position,
+                ttl: d.defaultTTL,
             };
 
             // Process the args: Functions are executed, objects are assumed to
@@ -2461,5 +2496,7 @@ function rawrecordBuilder(type) {
 
 // PLEASE KEEP THIS LIST ALPHABETICAL!
 
-// CLOUDFLAREAPI:
+var CF_REDIRECT = rawrecordBuilder('CF_REDIRECT');
 var CF_SINGLE_REDIRECT = rawrecordBuilder('CLOUDFLAREAPI_SINGLE_REDIRECT');
+var CF_TEMP_REDIRECT = rawrecordBuilder('CF_TEMP_REDIRECT');
+var RP = rawrecordBuilder('RP');
