@@ -12,6 +12,7 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
 	"github.com/StackExchange/dnscontrol/v4/pkg/rfc4183"
+	"github.com/StackExchange/dnscontrol/v4/pkg/rtypecontrol"
 	"github.com/StackExchange/dnscontrol/v4/pkg/transform"
 	"github.com/robertkrimen/otto"              // load underscore js into vm by default
 	_ "github.com/robertkrimen/otto/underscore" // required by otto
@@ -51,7 +52,6 @@ func ExecuteJavaScript(file string, devMode bool, variables map[string]string) (
 
 // ExecuteJavascriptString accepts a string containing javascript and runs it, returning the resulting dnsConfig.
 func ExecuteJavascriptString(script []byte, devMode bool, variables map[string]string) (*models.DNSConfig, error) {
-
 	vm := otto.New()
 	l := loop.New(vm)
 
@@ -69,15 +69,26 @@ func ExecuteJavascriptString(script []byte, devMode bool, variables map[string]s
 		}
 	}
 
-	vm.Set("require", require)
-	vm.Set("REV", reverse)
-	vm.Set("REVCOMPAT", reverseCompat)
-	vm.Set("glob", listFiles) // used for require_glob()
-	vm.Set("PANIC", jsPanic)
+	// add functions to otto
+	functions := map[string]any{
+		"require":   require,
+		"REV":       reverse,
+		"REVCOMPAT": reverseCompat,
+		"glob":      listFiles, // used for require_glob()
+		"PANIC":     jsPanic,
+		"HASH":      hashFunc,
+	}
+	for name, fn := range functions {
+		if err := vm.Set(name, fn); err != nil {
+			return nil, err
+		}
+	}
 
 	// add cli variables to otto
 	for key, value := range variables {
-		vm.Set(key, value)
+		if err := vm.Set(key, value); err != nil {
+			return nil, err
+		}
 	}
 
 	helperJs := GetHelpers(devMode)
@@ -109,6 +120,16 @@ func ExecuteJavascriptString(script []byte, devMode bool, variables map[string]s
 	if err = json.Unmarshal([]byte(str), conf); err != nil {
 		return nil, err
 	}
+
+	err = conf.PostProcess()
+	if err != nil {
+		return nil, err
+	}
+
+	if err := rtypecontrol.ImportRawRecords(conf.Domains); err != nil {
+		return nil, err
+	}
+
 	return conf, nil
 }
 
@@ -149,15 +170,14 @@ func require(call otto.FunctionCall) otto.Value {
 	printer.Debugf("requiring: %s (%s)\n", file, relFile)
 	// quick fix, by replacing to linux slashes, to make it work with windows paths too.
 	data, err := os.ReadFile(filepath.ToSlash(relFile))
-
 	if err != nil {
 		throw(call.Otto, err.Error())
 	}
 
-	var value = otto.TrueValue()
+	value := otto.TrueValue()
 
 	// If its a json file return the json value, else default to true
-	var ext = strings.ToLower(filepath.Ext(relFile))
+	ext := strings.ToLower(filepath.Ext(relFile))
 	if strings.HasSuffix(ext, "json") || strings.HasSuffix(ext, "json5") {
 		cmd := fmt.Sprintf(`JSON.parse(JSON.stringify(%s))`, string(data))
 		value, err = call.Otto.Run(cmd)
@@ -200,7 +220,7 @@ func listFiles(call otto.FunctionCall) otto.Value {
 	}
 
 	// Second: Recursive?
-	var recursive = true
+	recursive := true
 	if call.Argument(1).IsDefined() && !call.Argument(1).IsNull() {
 		if call.Argument(1).IsBoolean() {
 			recursive, _ = call.Argument(1).ToBoolean() // If it should be recursive
@@ -210,7 +230,7 @@ func listFiles(call otto.FunctionCall) otto.Value {
 	}
 
 	// Third: File extension filter.
-	var fileExtension = ".js"
+	fileExtension := ".js"
 	if call.Argument(2).IsDefined() && !call.Argument(2).IsNull() {
 		if call.Argument(2).IsString() {
 			fileExtension = call.Argument(2).String() // Which file extension to filter for.
@@ -246,7 +266,7 @@ func listFiles(call otto.FunctionCall) otto.Value {
 			// ONLY skip, when the file extension is NOT matching, or when filter is NOT disabled.
 			return nil
 		}
-		//dirPath := filepath.ToSlash(filepath.Dir(path)) + "/"
+		// dirPath := filepath.ToSlash(filepath.Dir(path)) + "/"
 		files = append(files, path)
 		return err
 	})
