@@ -6,23 +6,28 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"strings"
 	"testing"
 	"unicode"
 
+	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/normalize"
 	"github.com/StackExchange/dnscontrol/v4/pkg/prettyzone"
-	"github.com/StackExchange/dnscontrol/v4/providers"
-	_ "github.com/StackExchange/dnscontrol/v4/providers/_all"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
+	_ "github.com/StackExchange/dnscontrol/v4/pkg/providers/_all"
+	_ "github.com/StackExchange/dnscontrol/v4/pkg/rtype"
 	testifyrequire "github.com/stretchr/testify/require"
 )
 
 const (
-	testDir  = "pkg/js/parse_tests"
-	errorDir = "pkg/js/error_tests"
+	testDir = "pkg/js/parse_tests"
 )
 
 func init() {
-	os.Chdir("../..") // go up a directory so we helpers.js is in a consistent place.
+	// go up a directory so we helpers.js is in a consistent place.
+	if err := os.Chdir("../.."); err != nil {
+		panic(err)
+	}
 }
 
 func TestParsedFiles(t *testing.T) {
@@ -45,13 +50,25 @@ func TestParsedFiles(t *testing.T) {
 			if err != nil {
 				t.Fatal(err)
 			}
-			// for _, dc := range conf.Domains {
-			// 	normalize.UpdateNameSplitHorizon(dc)
-			// }
+
+			errs := normalize.ValidateAndNormalizeConfig(conf)
+			if len(errs) != 0 {
+				t.Fatal(errs[0])
+			}
+
+			for _, dc := range conf.Domains {
+				// fmt.Printf("DEBUG: PrettySort: domain=%q #rec=%d\n", dc.Name, len(dc.Records))
+				// fmt.Printf("DEBUG: records = %d %v\n", len(dc.Records), dc.Records)
+				ps := prettyzone.PrettySort(dc.Records, dc.Name, 0, nil)
+				dc.Records = ps.Records
+				if len(dc.Records) == 0 {
+					dc.Records = models.Records{}
+				}
+			}
 
 			// Initialize any DNS providers mentioned.
 			for _, dProv := range conf.DNSProviders {
-				var pcfg = map[string]string{}
+				pcfg := map[string]string{}
 
 				if dProv.Type == "-" {
 					// Pretend any "look up provider type in creds.json" results
@@ -85,20 +102,23 @@ func TestParsedFiles(t *testing.T) {
 			es := string(expectedJSON)
 			as := string(actualJSON)
 			_, _ = es, as
-			// When debugging, leave behind the actual result:
-			//os.WriteFile(expectedFile+".ACTUAL", []byte(as), 0644)
+			// Leave behind the actual result:
+			if err := os.WriteFile(expectedFile+".ACTUAL", []byte(as), 0o644); err != nil {
+				t.Fatal(err)
+			}
 			testifyrequire.JSONEqf(t, es, as, "EXPECTING %q = \n```\n%s\n```", expectedFile, as)
 
 			// For each domain, if there is a zone file, test against it:
 
-			errs := normalize.ValidateAndNormalizeConfig(conf)
-			if len(errs) != 0 {
-				t.Fatal(errs[0])
-			}
-
 			var dCount int
 			for _, dc := range conf.Domains {
-				zoneFile := filepath.Join(testDir, testName, dc.Name+".zone")
+				var zoneFile string
+				if dc.Tag != "" {
+					zoneFile = filepath.Join(testDir, testName, dc.GetUniqueName()+".zone")
+				} else {
+					zoneFile = filepath.Join(testDir, testName, dc.Name+".zone")
+				}
+				//fmt.Printf("DEBUG: zonefile = %q\n", zoneFile)
 				expectedZone, err := os.ReadFile(zoneFile)
 				if err != nil {
 					continue
@@ -113,20 +133,20 @@ func TestParsedFiles(t *testing.T) {
 				}
 				actualZone := buf.String()
 
-				es := string(expectedZone)
-				as := actualZone
+				es := strings.TrimSpace(string(expectedZone))
+				as := strings.TrimSpace(actualZone)
 				if es != as {
 					// On failure, leave behind the .ACTUAL file.
-					os.WriteFile(zoneFile+".ACTUAL", []byte(actualZone), 0644)
+					if err := os.WriteFile(zoneFile+".ACTUAL", []byte(actualZone), 0o644); err != nil {
+						t.Fatal(err)
+					}
 				}
 				testifyrequire.Equal(t, es, as, "EXPECTING %q =\n```\n%s```", zoneFile, as)
 			}
 			if dCount > 0 && (len(conf.Domains) != dCount) {
 				t.Fatal(fmt.Errorf("only %d of %d domains in %q have zonefiles", dCount, len(conf.Domains), name))
 			}
-
 		})
-
 	}
 }
 
@@ -138,9 +158,12 @@ func TestErrors(t *testing.T) {
 		{"CF_REDIRECT With comma", `D("foo.com","reg",CF_REDIRECT("foo.com,","baaa"))`},
 		{"CF_TEMP_REDIRECT With comma", `D("foo.com","reg",CF_TEMP_REDIRECT("foo.com","baa,a"))`},
 		{"CF_WORKER_ROUTE With comma", `D("foo.com","reg",CF_WORKER_ROUTE("foo.com","baa,a"))`},
+		{"ADGUARDHOME_A_PASSTHROUGH With non-empty value", `D("foo.com","reg",ADGUARDHOME_A_PASSTHROUGH("foo","baaa"))`},
+		{"ADGUARDHOME_AAAA_PASSTHROUGH With non-empty value", `D("foo.com","reg",ADGUARDHOME_AAAA_PASSTHROUGH("foo,","baaa"))`},
 		{"Bad cidr", `D(reverse("foo.com"), "reg")`},
 		{"Dup domains", `D("example.org", "reg"); D("example.org", "reg")`},
 		{"Bad NAMESERVER", `D("example.com","reg", NAMESERVER("@","ns1.foo.com."))`},
+		{"Bad Hash function", `D(HASH("123", "abc"),"reg")`},
 	}
 	for _, tst := range tests {
 		t.Run(tst.desc, func(t *testing.T) {
@@ -148,6 +171,5 @@ func TestErrors(t *testing.T) {
 				t.Fatal("Expected error but found none")
 			}
 		})
-
 	}
 }
