@@ -1,14 +1,17 @@
 package normalize
 
 import (
+	"errors"
 	"fmt"
 	"net"
+	"slices"
 	"sort"
+	"strconv"
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	"github.com/StackExchange/dnscontrol/v4/pkg/transform"
-	"github.com/StackExchange/dnscontrol/v4/providers"
 	"github.com/miekg/dns"
 	"github.com/miekg/dns/dnsutil"
 )
@@ -35,7 +38,7 @@ func checkTarget(target string) error {
 		return nil
 	}
 	if target == "" {
-		return fmt.Errorf("empty target")
+		return errors.New("empty target (\"\"). Did you mean \"@\" instead?")
 	}
 	if strings.ContainsAny(target, `'" +,|!£$%&()=?^*ç°§;:<>[]()@`) {
 		return fmt.Errorf("target (%v) includes invalid char", target)
@@ -52,8 +55,13 @@ func checkTarget(target string) error {
 
 // validateRecordTypes list of valid rec.Type values. Returns true if this is a real DNS record type, false means it is a pseudo-type used internally.
 func validateRecordTypes(rec *models.RecordConfig, domain string, pTypes []string) error {
+	if rec.IsModernType() {
+		// Modern types do their own validation.
+		return nil
+	}
+
 	// #rtype_variations
-	var validTypes = map[string]bool{
+	validTypes := map[string]bool{
 		"A":                true,
 		"AAAA":             true,
 		"ALIAS":            false,
@@ -69,7 +77,9 @@ func validateRecordTypes(rec *models.RecordConfig, domain string, pTypes []strin
 		"MX":               true,
 		"NAPTR":            true,
 		"NS":               true,
+		"OPENPGPKEY":       true,
 		"PTR":              true,
+		"SMIMEA":           true,
 		"SOA":              true,
 		"SRV":              true,
 		"SSHFP":            true,
@@ -112,14 +122,14 @@ func checkLabel(label string, rType string, domain string, meta map[string]strin
 		return nil
 	}
 	if label == "" {
-		return fmt.Errorf("empty %s label in %s", rType, domain)
+		return fmt.Errorf("empty %s label (\"\") in %s. Did you mean \"@\" instead?", rType, domain)
 	}
 	if label[len(label)-1] == '.' {
 		return fmt.Errorf("label %s.%s ends with a (.)", label, domain)
 	}
 	if label == domain || strings.HasSuffix(label, "."+domain) {
 		if m := meta["skip_fqdn_check"]; m != "true" {
-			return fmt.Errorf(errorRepeat(label, domain))
+			return errors.New(errorRepeat(label, domain))
 		}
 	}
 
@@ -127,10 +137,8 @@ func checkLabel(label string, rType string, domain string, meta map[string]strin
 	// are used in a way we consider typical.  Yes, we're opinionated here.
 
 	// Don't warn for certain rtypes:
-	for _, ex := range []string{"SRV", "TLSA", "TXT"} {
-		if rType == ex {
-			return nil
-		}
+	if slices.Contains([]string{"SRV", "TLSA", "TXT", "LUA"}, rType) {
+		return nil
 	}
 	// Don't warn for records that start with _
 	// See https://github.com/StackExchange/dnscontrol/issues/829
@@ -148,33 +156,38 @@ func checkLabel(label string, rType string, domain string, meta map[string]strin
 
 func checkSoa(expire uint32, minttl uint32, refresh uint32, retry uint32, mbox string) error {
 	if expire <= 0 {
-		return fmt.Errorf("SOA Expire must be > 0")
+		return errors.New("SOA Expire must be > 0")
 	}
 	if minttl <= 0 {
-		return fmt.Errorf("SOA Minimum TTL must be > 0")
+		return errors.New("SOA Minimum TTL must be > 0")
 	}
 	if refresh <= 0 {
-		return fmt.Errorf("SOA Refresh must be > 0")
+		return errors.New("SOA Refresh must be > 0")
 	}
 	if retry <= 0 {
-		return fmt.Errorf("SOA Retry must be > 0")
+		return errors.New("SOA Retry must be > 0")
 	}
 	if mbox == "" {
-		return fmt.Errorf("SOA MBox must be specified")
+		return errors.New("SOA MBox must be specified")
 	}
 	if strings.ContainsRune(mbox, '@') {
-		return fmt.Errorf("SOA MBox must have '.' instead of '@'")
+		return errors.New("SOA MBox must have '.' instead of '@'")
 	}
 	return nil
 }
 
 // checkTargets returns true if rec.Target is valid for the rec.Type.
 func checkTargets(rec *models.RecordConfig, domain string) (errs []error) {
+	if rec.IsModernType() {
+		// Modern types do their own validation.
+		return nil
+	}
+
 	label := rec.GetLabel()
 	target := rec.GetTargetField()
 	check := func(e error) {
 		if e != nil {
-			err := fmt.Errorf("in %s %s.%s: %s", rec.Type, rec.GetLabel(), domain, e.Error())
+			err := fmt.Errorf("%s: %s %s.%s: %s", rec.FilePos, rec.Type, rec.GetLabel(), domain, e.Error())
 			if _, ok := e.(Warning); ok {
 				err = Warning{err}
 			}
@@ -191,12 +204,12 @@ func checkTargets(rec *models.RecordConfig, domain string) (errs []error) {
 	case "CNAME":
 		check(checkTarget(target))
 		if label == "@" {
-			check(fmt.Errorf("cannot create CNAME record for bare domain"))
+			check(errors.New("cannot create CNAME record for bare domain"))
 		}
 		labelFQDN := dnsutil.AddOrigin(label, domain)
 		targetFQDN := dnsutil.AddOrigin(target, domain)
 		if labelFQDN == targetFQDN {
-			check(fmt.Errorf("CNAME loop (target points at itself)"))
+			check(errors.New("CNAME loop (target points at itself)"))
 		}
 	case "DNAME":
 		check(checkTarget(target))
@@ -210,11 +223,7 @@ func checkTargets(rec *models.RecordConfig, domain string) (errs []error) {
 	case "NS":
 		check(checkTarget(target))
 		if label == "@" {
-			check(fmt.Errorf("cannot create NS record for bare domain. Use NAMESERVER instead"))
-		}
-	case "NS1_URLFWD":
-		if len(strings.Fields(target)) != 5 {
-			check(fmt.Errorf("record should follow format: \"from to redirectType pathForwardingMode queryForwarding\""))
+			check(errors.New("cannot create NS record for bare domain. Use NAMESERVER instead"))
 		}
 	case "PTR":
 		check(checkTarget(target))
@@ -222,33 +231,66 @@ func checkTargets(rec *models.RecordConfig, domain string) (errs []error) {
 		check(checkSoa(rec.SoaExpire, rec.SoaMinttl, rec.SoaRefresh, rec.SoaRetry, rec.SoaMbox))
 		check(checkTarget(target))
 		if label != "@" {
-			check(fmt.Errorf("SOA record is only valid for bare domain"))
+			check(errors.New("SOA record is only valid for bare domain"))
 		}
 	case "SRV":
 		check(checkTarget(target))
-	case "CAA", "DHCID", "DNSKEY", "DS", "HTTPS", "IMPORT_TRANSFORM", "SSHFP", "SVCB", "TLSA", "TXT":
+	case "LUA":
+		upper := strings.ToUpper(rec.LuaRType)
+		if upper == "" {
+			check(errors.New("LUA records must specify an emitted rtype"))
+			break
+		}
+		if _, ok := dns.StringToType[upper]; !ok {
+			check(fmt.Errorf("LUA emitted rtype (%s) is not a valid DNS type", rec.LuaRType))
+		}
+		rec.LuaRType = upper
+	case "CAA", "DHCID", "DNSKEY", "DS", "HTTPS", "IMPORT_TRANSFORM", "OPENPGPKEY", "SMIMEA", "SSHFP", "SVCB", "TLSA", "TXT":
 	default:
 		if rec.Metadata["orig_custom_type"] != "" {
 			// it is a valid custom type. We perform no validation on target
-			return
+			return errs
 		}
 		errs = append(errs, fmt.Errorf("checkTargets: Unimplemented record type (%v) domain=%v name=%v",
 			rec.Type, domain, rec.GetLabel()))
 	}
-	return
+	return errs
 }
 
-func transformCNAME(target, oldDomain, newDomain string) string {
-	// Canonicalize. If it isn't a FQDN, add the newDomain.
-	result := dnsutil.AddOrigin(target, oldDomain)
-	if dns.IsFqdn(result) {
-		result = result[:len(result)-1]
+func transformCNAME(target, oldDomain, newDomain, suffixstrip string) string {
+	// Canonicalize the target.  Add the newDomain minus the suffixstrip.
+	//  foo -> foo.oldDomain.newDomain
+	//  foo. -> foo.newDomain
+	nd := strings.TrimPrefix(newDomain, suffixstrip+".")
+	if strings.HasSuffix(target, ".") {
+		return target + nd + "."
 	}
-	return dnsutil.AddOrigin(result, newDomain) + "."
+	return dnsutil.AddOrigin(target, oldDomain) + "." + nd + "."
+}
+
+func newRec(rec *models.RecordConfig, ttl uint32) *models.RecordConfig {
+	rec2, _ := rec.Copy()
+	if ttl != 0 {
+		rec2.TTL = ttl
+	}
+	return rec2
+}
+
+func transformLabel(label, suffixstrip string) (string, error) {
+	if suffixstrip == "" {
+		return label, nil
+	}
+	suffixstrip = "." + suffixstrip
+	if !strings.HasSuffix(label, suffixstrip) {
+		return "", fmt.Errorf("label %q does not end with %q", label, suffixstrip)
+	}
+	return label[:len(label)-len(suffixstrip)], nil
 }
 
 // import_transform imports the records of one zone into another, modifying records along the way.
-func importTransform(srcDomain, dstDomain *models.DomainConfig, transforms []transform.IPConversion, ttl uint32) error {
+func importTransform(srcDomain, dstDomain *models.DomainConfig,
+	transforms []transform.IPConversion, ttl uint32, suffixstrip string,
+) error {
 	// Read srcDomain.Records, transform, and append to dstDomain.Records:
 	// 1. Skip any that aren't A or CNAMEs.
 	// 2. Append destDomainname to the end of the label.
@@ -256,32 +298,42 @@ func importTransform(srcDomain, dstDomain *models.DomainConfig, transforms []tra
 	// 4. For As, change the target as described the transforms.
 
 	for _, rec := range srcDomain.Records {
-		if dstDomain.Records.HasRecordTypeName(rec.Type, rec.GetLabelFQDN()) {
+		// If this record is marked to be skipped, skip it.
+		if rec.Metadata["import_transform_skip"] != "" {
 			continue
 		}
-		newRec := func() *models.RecordConfig {
-			rec2, _ := rec.Copy()
-			newlabel := rec2.GetLabelFQDN()
-			rec2.SetLabel(newlabel, dstDomain.Name)
-			if ttl != 0 {
-				rec2.TTL = ttl
-			}
-			return rec2
+		// If the dstDomain already has a record with this type+label, skip it.
+		if dstDomain.Records.HasRecordTypeName(rec.Type, rec.GetLabelFQDN()) {
+			continue
 		}
 		switch rec.Type {
 		case "A":
 			trs, err := transform.IPToList(net.ParseIP(rec.GetTargetField()), transforms)
 			if err != nil {
-				return fmt.Errorf("import_transform: TransformIP(%v, %v) returned err=%s", rec.GetTargetField(), transforms, err)
+				return fmt.Errorf("import_transform: TransformIP(%v, %v) returned err=%w", rec.GetTargetField(), transforms, err)
 			}
 			for _, tr := range trs {
-				r := newRec()
-				r.SetTarget(tr.String())
+				r := newRec(rec, ttl)
+				l, err := transformLabel(r.GetLabelFQDN(), suffixstrip)
+				if err != nil {
+					return err
+				}
+				r.SetLabel(l, dstDomain.Name)
+				if err := r.SetTarget(tr.String()); err != nil {
+					return err
+				}
 				dstDomain.Records = append(dstDomain.Records, r)
 			}
 		case "CNAME":
-			r := newRec()
-			r.SetTarget(transformCNAME(r.GetTargetField(), srcDomain.Name, dstDomain.Name))
+			r := newRec(rec, ttl)
+			l, err := transformLabel(r.GetLabelFQDN(), suffixstrip)
+			if err != nil {
+				return err
+			}
+			r.SetLabel(l, dstDomain.Name)
+			if err := r.SetTarget(transformCNAME(r.GetTargetField(), srcDomain.Name, dstDomain.Name, suffixstrip)); err != nil {
+				return err
+			}
 			dstDomain.Records = append(dstDomain.Records, r)
 		default:
 			// Anything else is ignored.
@@ -348,7 +400,6 @@ func ValidateAndNormalizeConfig(config *models.DNSConfig) (errs []error) {
 		// Normalize Records.
 		models.PostProcessRecords(domain.Records)
 		for _, rec := range domain.Records {
-
 			if rec.TTL == 0 {
 				rec.TTL = models.DefaultTTL
 			}
@@ -397,9 +448,13 @@ func ValidateAndNormalizeConfig(config *models.DNSConfig) (errs []error) {
 				if rec.SubDomain != "" {
 					origin = rec.SubDomain + "." + origin
 				}
-				rec.SetTarget(dnsutil.AddOrigin(rec.GetTargetField(), origin))
+				if err := rec.SetTarget(dnsutil.AddOrigin(rec.GetTargetField(), origin)); err != nil {
+					errs = append(errs, err)
+				}
 			} else if rec.Type == "A" || rec.Type == "AAAA" {
-				rec.SetTarget(net.ParseIP(rec.GetTargetField()).String())
+				if err := rec.SetTarget(net.ParseIP(rec.GetTargetField()).String()); err != nil {
+					errs = append(errs, err)
+				}
 			} else if rec.Type == "PTR" {
 				var err error
 				var name string
@@ -408,7 +463,9 @@ func ValidateAndNormalizeConfig(config *models.DNSConfig) (errs []error) {
 				}
 				rec.SetLabel(name, domain.Name)
 			} else if rec.Type == "CAA" {
-				if rec.CaaTag != "issue" && rec.CaaTag != "issuewild" && rec.CaaTag != "iodef" {
+				// Per: https://www.iana.org/assignments/pkix-parameters/pkix-parameters.xhtml#caa-properties excluding reserved tags
+				allowedTags := []string{"issue", "issuewild", "iodef", "contactemail", "contactphone", "issuemail", "issuevmc"}
+				if !slices.Contains(allowedTags, rec.CaaTag) {
 					errs = append(errs, fmt.Errorf("CAA tag %s is invalid", rec.CaaTag))
 				}
 			} else if rec.Type == "TLSA" {
@@ -424,15 +481,27 @@ func ValidateAndNormalizeConfig(config *models.DNSConfig) (errs []error) {
 					errs = append(errs, fmt.Errorf("TLSA MatchingType %d is invalid in record %s (domain %s)",
 						rec.TlsaMatchingType, rec.GetLabel(), domain.Name))
 				}
+			} else if rec.Type == "SMIMEA" {
+				if rec.SmimeaUsage > 3 {
+					errs = append(errs, fmt.Errorf("SMIMEA Usage %d is invalid in record %s (domain %s)",
+						rec.SmimeaUsage, rec.GetLabel(), domain.Name))
+				}
+				if rec.SmimeaSelector > 1 {
+					errs = append(errs, fmt.Errorf("SMIMEA Selector %d is invalid in record %s (domain %s)",
+						rec.SmimeaSelector, rec.GetLabel(), domain.Name))
+				}
+				if rec.SmimeaMatchingType > 2 {
+					errs = append(errs, fmt.Errorf("SMIMEA MatchingType %d is invalid in record %s (domain %s)",
+						rec.SmimeaMatchingType, rec.GetLabel(), domain.Name))
+				}
 			}
 
 			// Populate FQDN:
 			rec.SetLabel(rec.GetLabel(), domain.Name)
 
 			if _, ok := rec.Metadata["ignore_name_disable_safety_check"]; ok {
-				errs = append(errs, fmt.Errorf("IGNORE_NAME_DISABLE_SAFETY_CHECK no longer supported. Please use DISABLE_IGNORE_SAFETY_CHECK for the entire domain"))
+				errs = append(errs, errors.New("IGNORE_NAME_DISABLE_SAFETY_CHECK no longer supported. Please use DISABLE_IGNORE_SAFETY_CHECK for the entire domain"))
 			}
-
 		}
 	}
 
@@ -445,6 +514,7 @@ func ValidateAndNormalizeConfig(config *models.DNSConfig) (errs []error) {
 	for _, domain := range config.Domains {
 		for _, rec := range domain.Records {
 			if rec.Type == "IMPORT_TRANSFORM" {
+				suffixstrip := rec.Metadata["transform_suffixstrip"]
 				table, err := transform.DecodeTransformTable(rec.Metadata["transform_table"])
 				if err != nil {
 					errs = append(errs, err)
@@ -452,10 +522,10 @@ func ValidateAndNormalizeConfig(config *models.DNSConfig) (errs []error) {
 				}
 				c := config.FindDomain(rec.GetTargetField())
 				if c == nil {
-					err = fmt.Errorf("IMPORT_TRANSFORM mentions non-existant domain %q", rec.GetTargetField())
+					err = fmt.Errorf("IMPORT_TRANSFORM mentions non-existent domain %q", rec.GetTargetField())
 					errs = append(errs, err)
 				}
-				err = importTransform(c, domain, table, rec.TTL)
+				err = importTransform(c, domain, table, rec.TTL, suffixstrip)
 				if err != nil {
 					errs = append(errs, err)
 				}
@@ -522,15 +592,13 @@ func ValidateAndNormalizeConfig(config *models.DNSConfig) (errs []error) {
 
 // processSplitHorizonDomains finds "domain.tld!tag" domains and pre-processes them.
 func processSplitHorizonDomains(config *models.DNSConfig) error {
-	// Parse out names and tags.
-	for _, d := range config.Domains {
-		d.UpdateSplitHorizonNames()
-	}
 
 	// Verify uniquenames are unique
 	seen := map[string]bool{}
 	for _, d := range config.Domains {
 		uniquename := d.GetUniqueName()
+		// empty tag == untagged ("example.com!" -> "example.com")
+		uniquename = strings.TrimSuffix(uniquename, "!")
 		if seen[uniquename] {
 			return fmt.Errorf("duplicate domain name: %q", uniquename)
 		}
@@ -559,14 +627,14 @@ func checkCNAMEs(dc *models.DomainConfig) (errs []error) {
 	for _, r := range dc.Records {
 		if r.Type == "CNAME" {
 			if cnames[r.GetLabel()] {
-				errs = append(errs, fmt.Errorf("cannot have multiple CNAMEs with same name: %s", r.GetLabelFQDN()))
+				errs = append(errs, fmt.Errorf("%s: cannot have multiple CNAMEs with same name: %s", r.FilePos, r.GetLabelFQDN()))
 			}
 			cnames[r.GetLabel()] = true
 		}
 	}
 	for _, r := range dc.Records {
 		if cnames[r.GetLabel()] && r.Type != "CNAME" {
-			errs = append(errs, fmt.Errorf("cannot have CNAME and %s record with same name: %s", r.Type, r.GetLabelFQDN()))
+			errs = append(errs, fmt.Errorf("%s: cannot have CNAME and %s record with same name: %s", r.FilePos, r.Type, r.GetLabelFQDN()))
 		}
 	}
 	return
@@ -648,7 +716,6 @@ func checkRecordSetHasMultipleTTLs(records []*models.RecordConfig) (errs []error
 func formatInconsistency(r map[string]map[uint32]bool) string {
 	var rtypeResult []string
 	for rtype, ttlsMap := range r {
-
 		ttlList := make([]int, len(ttlsMap))
 		i := 0
 		for k := range ttlsMap {
@@ -667,7 +734,7 @@ func formatInconsistency(r map[string]map[uint32]bool) string {
 func commaSepInts(list []int) string {
 	slist := make([]string, len(list))
 	for i, v := range list {
-		slist[i] = fmt.Sprintf("%d", v)
+		slist[i] = strconv.Itoa(v)
 	}
 	return strings.Join(slist, ",")
 }
@@ -678,8 +745,9 @@ func commaSepInts(list []int) string {
 var providerCapabilityChecks = []pairTypeCapability{
 	// #rtype_variations
 	// If a zone uses rType X, the provider must support capability Y.
-	//{"X", providers.Y},
+	// {"X", providers.Y},
 	capabilityCheck("AKAMAICDN", providers.CanUseAKAMAICDN),
+	capabilityCheck("AKAMAITLC", providers.CanUseAKAMAITLC),
 	capabilityCheck("ALIAS", providers.CanUseAlias),
 	capabilityCheck("AUTODNSSEC", providers.CanAutoDNSSEC),
 	capabilityCheck("AZURE_ALIAS", providers.CanUseAzureAlias),
@@ -690,8 +758,11 @@ var providerCapabilityChecks = []pairTypeCapability{
 	capabilityCheck("HTTPS", providers.CanUseHTTPS),
 	capabilityCheck("LOC", providers.CanUseLOC),
 	capabilityCheck("NAPTR", providers.CanUseNAPTR),
+	capabilityCheck("OPENPGPKEY", providers.CanUseOPENPGPKEY),
 	capabilityCheck("PTR", providers.CanUsePTR),
 	capabilityCheck("R53_ALIAS", providers.CanUseRoute53Alias),
+	capabilityCheck("RP", providers.CanUseRP),
+	capabilityCheck("SMIMEA", providers.CanUseSMIMEA),
 	capabilityCheck("SOA", providers.CanUseSOA),
 	capabilityCheck("SRV", providers.CanUseSRV),
 	capabilityCheck("SSHFP", providers.CanUseSSHFP),
@@ -774,7 +845,6 @@ func checkProviderCapabilities(dc *models.DomainConfig) error {
 					break
 				}
 			}
-
 		}
 		if !hasAny {
 			continue
@@ -825,15 +895,20 @@ func applyRecordTransforms(domain *models.DomainConfig) error {
 		}
 		for i, newIP := range newIPs {
 			if i == 0 && !newIP.Equal(ip) {
-				rec.SetTarget(newIP.String()) // replace target of first record if different
+				// replace target of first record if different
+				if err := rec.SetTarget(newIP.String()); err != nil {
+					return err
+				}
 			} else if i > 0 {
 				// any additional ips need identical records with the alternate ip added to the domain
-				copy, err := rec.Copy()
+				cpy, err := rec.Copy()
 				if err != nil {
 					return err
 				}
-				copy.SetTarget(newIP.String())
-				domain.Records = append(domain.Records, copy)
+				if err := cpy.SetTarget(newIP.String()); err != nil {
+					return err
+				}
+				domain.Records = append(domain.Records, cpy)
 			}
 		}
 	}
