@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"io"
 	"sort"
+	"strconv"
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
@@ -87,7 +88,6 @@ func PrettySort(records models.Records, origin string, defaultTTL uint32, commen
 
 // generateZoneFileHelper creates a pretty zonefile.
 func (z *ZoneGenData) generateZoneFileHelper(w io.Writer) error {
-
 	nameShortPrevious := ""
 
 	sort.Sort(z)
@@ -96,14 +96,13 @@ func (z *ZoneGenData) generateZoneFileHelper(w io.Writer) error {
 	}
 	fmt.Fprintln(w, "$TTL", z.DefaultTTL)
 	for _, comment := range z.Comments {
-		for _, line := range strings.Split(comment, "\n") {
+		for line := range strings.SplitSeq(comment, "\n") {
 			if line != "" {
 				fmt.Fprintln(w, ";", line)
 			}
 		}
 	}
 	for i, rr := range z.Records {
-
 		// Fake types are commented out.
 		prefix := ""
 		_, ok := dns.StringToType[rr.Type]
@@ -122,7 +121,7 @@ func (z *ZoneGenData) generateZoneFileHelper(w io.Writer) error {
 		// ttl
 		ttl := ""
 		if rr.TTL != z.DefaultTTL && rr.TTL != 0 {
-			ttl = fmt.Sprint(rr.TTL)
+			ttl = strconv.FormatUint(uint64(rr.TTL), 10)
 		}
 
 		// type
