@@ -27,15 +27,16 @@ func (z *ZoneGenData) Less(i, j int) bool {
 
 	// Sort by name.
 
-	// If we are at the apex, use "@" in the sorting.
+	//fmt.Printf("DEBUG: LabelLess(%q, %q) = %v %q %q\n", compA, compB, LabelLess(compA, compB), a.Name, b.Name)
 	compA, compB := a.NameFQDN, b.NameFQDN
+	// Unify FQDNs to "@". LabelLess needs FQDNs to be "@" to work properly.
+	if a.Name == "@" {
+		compA = "@"
+	}
+	if b.Name == "@" {
+		compB = "@"
+	}
 	if compA != compB {
-		if a.Name == "@" {
-			compA = "@"
-		}
-		if b.Name == "@" {
-			compB = "@"
-		}
 		return LabelLess(compA, compB)
 	}
 
@@ -67,7 +68,7 @@ func (z *ZoneGenData) Less(i, j int) bool {
 		}
 		return a.MxPreference < b.MxPreference
 	case "SRV":
-		//ta2, tb2 := a.(*dns.SRV), b.(*dns.SRV)
+		// ta2, tb2 := a.(*dns.SRV), b.(*dns.SRV)
 		pa, pb := a.SrvPort, b.SrvPort
 		if pa != pb {
 			return pa < pb
@@ -87,13 +88,13 @@ func (z *ZoneGenData) Less(i, j int) bool {
 		}
 		return a.SvcPriority < b.SvcPriority
 	case "PTR":
-		//ta2, tb2 := a.(*dns.PTR), b.(*dns.PTR)
+		// ta2, tb2 := a.(*dns.PTR), b.(*dns.PTR)
 		pa, pb := a.GetTargetField(), b.GetTargetField()
 		if pa != pb {
 			return pa < pb
 		}
 	case "CAA":
-		//ta2, tb2 := a.(*dns.CAA), b.(*dns.CAA)
+		// ta2, tb2 := a.(*dns.CAA), b.(*dns.CAA)
 		// sort by tag
 		pa, pb := a.CaaTag, b.CaaTag
 		if pa != pb {
@@ -122,6 +123,7 @@ func (z *ZoneGenData) Less(i, j int) bool {
 	default:
 		// pass through. String comparison is sufficient.
 	}
+	// fmt.Printf("DEBUG: Less %q < %q == %v\n", a.String(), b.String(), a.String() < b.String())
 	return a.String() < b.String()
 }
 
@@ -130,14 +132,13 @@ func (z *ZoneGenData) Less(i, j int) bool {
 func LabelLess(a, b string) bool {
 	// Compare two zone labels for the purpose of sorting the RRs in a Zone.
 
-	// If they are equal, we are done. All other code is simplified
-	// because we can assume a!=b.
+	// If they are equal, we are done. The remainingi code can assume a != b.
 	if a == b {
 		return false
 	}
 
 	// Sort @ at the top, then *, then everything else lexigraphically.
-	// i.e. @ always is less. * is is less than everything but @.
+	// i.e. @ always is less. * is less than everything but @.
 	if a == "@" {
 		return true
 	}
@@ -159,22 +160,21 @@ func LabelLess(a, b string) bool {
 	ia := len(as) - 1
 	ib := len(bs) - 1
 
-	var min int
+	var minIdx int
 	if ia < ib {
-		min = len(as) - 1
+		minIdx = len(as) - 1
 	} else {
-		min = len(bs) - 1
+		minIdx = len(bs) - 1
 	}
 
 	// Skip the matching highest elements, then compare the next item.
-	for i, j := ia, ib; min >= 0; i, j, min = i-1, j-1, min-1 {
+	for i, j := ia, ib; minIdx >= 0; i, j, minIdx = i-1, j-1, minIdx-1 {
 		// Compare as[i] < bs[j]
 		// Sort @ at the top, then *, then everything else.
-		// i.e. @ always is less. * is is less than everything but @.
+		// i.e. @ always is less. * is less than everything but @.
 		// If both are numeric, compare as integers, otherwise as strings.
 
 		if as[i] != bs[j] {
-
 			// If the first element is *, it is always less.
 			if i == 0 && as[i] == "*" {
 				return true
@@ -206,8 +206,9 @@ func zoneRrtypeLess(a, b string) bool {
 
 	// List SOAs, NSs, etc. then all others alphabetically.
 
-	for _, t := range []string{"SOA", "NS", "CNAME",
-		"A", "AAAA", "MX", "SRV", "TXT",
+	for _, t := range []string{
+		"SOA", "NS", "CNAME",
+		"A", "AAAA", "MX", "SRV", "TXT", "LUA",
 	} {
 		if a == t {
 			return true
