@@ -4,9 +4,13 @@ import (
 	"bytes"
 	"encoding/json"
 	"errors"
+	"fmt"
 	"io"
+	"math"
 	"net/http"
+	"os"
 	"sort"
+	"time"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 )
@@ -25,7 +29,15 @@ type ZoneListRequest struct {
 	Filter []*ZoneListFilter `json:"filters"`
 }
 
-func (api *autoDNSProvider) request(method string, requestPath string, data interface{}) ([]byte, error) {
+var (
+	// Default retry configuration
+	defaultRetryWait = 3 * time.Second
+	defaultRetryMax  = 4
+)
+
+func (api *autoDNSProvider) request(method string, requestPath string, data any) ([]byte, error) {
+	var retryCounter = 0
+
 	client := &http.Client{}
 
 	requestURL := api.baseURL
@@ -43,18 +55,36 @@ func (api *autoDNSProvider) request(method string, requestPath string, data inte
 		request.Body = io.NopCloser(buffer)
 	}
 
-	response, error := client.Do(request)
-	if error != nil {
-		return nil, error
-	}
-	defer response.Body.Close()
+	for {
+		response, err := client.Do(request)
+		if err != nil {
+			return nil, err
+		}
+		defer response.Body.Close()
 
-	responseText, _ := io.ReadAll(response.Body)
-	if response.StatusCode != 200 {
-		return nil, errors.New("Request to " + requestURL.Path + " failed: " + string(responseText))
+		responseText, _ := io.ReadAll(response.Body)
+
+		// FUTUREWORK: 202 starts a long-running task. Should we instead poll here until task is completed?
+		if response.StatusCode != http.StatusOK && response.StatusCode != http.StatusAccepted && response.StatusCode != http.StatusTooManyRequests {
+			return nil, errors.New("Request to " + requestURL.Path + " failed: " + string(responseText))
+		}
+
+		if response.StatusCode == http.StatusOK {
+			return responseText, nil
+		}
+
+		if retryCounter == defaultRetryMax { // the condition stops matching
+			break // break out of the loop
+		}
+
+		retryCounter++
+
+		sleepDuration := time.Duration(math.Pow(2, float64(retryCounter)) * float64(defaultRetryWait))
+
+		time.Sleep(sleepDuration)
 	}
 
-	return responseText, nil
+	return nil, errors.New("Failed to fetch" + requestURL.Path + " after 4 retries")
 }
 
 func (api *autoDNSProvider) findZoneSystemNameServer(domain string) (*models.Nameserver, error) {
@@ -72,9 +102,12 @@ func (api *autoDNSProvider) findZoneSystemNameServer(domain string) (*models.Nam
 	}
 
 	var responseObject JSONResponseDataZone
-	_ = json.Unmarshal(responseData, &responseObject)
+	if err := json.Unmarshal(responseData, &responseObject); err != nil {
+		return nil, err
+	}
+
 	if len(responseObject.Data) != 1 {
-		return nil, errors.New("Domain " + domain + " could not be found in AutoDNS")
+		return nil, fmt.Errorf("Zone "+domain+" could not be found in AutoDNS: %w", os.ErrNotExist)
 	}
 
 	systemNameServer := &models.Nameserver{Name: responseObject.Data[0].SystemNameServer}
@@ -82,6 +115,42 @@ func (api *autoDNSProvider) findZoneSystemNameServer(domain string) (*models.Nam
 	return systemNameServer, nil
 }
 
+func (api *autoDNSProvider) createZone(domain string, zone *Zone) (*Zone, error) {
+	responseData, err := api.request("POST", "zone", zone)
+	if err != nil {
+		return nil, err
+	}
+
+	var responseObject JSONResponseDataZone
+	if err := json.Unmarshal(responseData, &responseObject); err != nil {
+		return nil, err
+	}
+
+	if len(responseObject.Data) != 1 {
+		return nil, errors.New("Zone " + domain + " not returned")
+	}
+
+	return responseObject.Data[0], nil
+}
+
+func (api *autoDNSProvider) getDomain(domain string) (*Domain, error) {
+	responseData, err := api.request("GET", "domain/"+domain, nil)
+	if err != nil {
+		return nil, err
+	}
+
+	var responseObject JSONResponseDataDomain
+	if err := json.Unmarshal(responseData, &responseObject); err != nil {
+		return nil, err
+	}
+
+	if len(responseObject.Data) != 1 {
+		return nil, fmt.Errorf("Domain "+domain+" could not be found in AutoDNS: %w", os.ErrNotExist)
+	}
+
+	return responseObject.Data[0], nil
+}
+
 func (api *autoDNSProvider) getZone(domain string) (*Zone, error) {
 	systemNameServer, err := api.findZoneSystemNameServer(domain)
 	if err != nil {
@@ -89,25 +158,51 @@ func (api *autoDNSProvider) getZone(domain string) (*Zone, error) {
 	}
 
 	// if resolving of a systemNameServer succeeds the system contains this zone
-	var responseData, _ = api.request("GET", "zone/"+domain+"/"+systemNameServer.Name, nil)
+	responseData, err2 := api.request("GET", "zone/"+domain+"/"+systemNameServer.Name, nil)
+
+	if err2 != nil {
+		return nil, err2
+	}
+
 	var responseObject JSONResponseDataZone
 	// make sure that the response is valid, the zone is in AutoDNS but we're not sure the returned data meets our expectation
-	unmErr := json.Unmarshal(responseData, &responseObject)
-	if unmErr != nil {
-		return nil, unmErr
+	if err := json.Unmarshal(responseData, &responseObject); err != nil {
+		return nil, err
 	}
 
 	return responseObject.Data[0], nil
 }
 
+func (api *autoDNSProvider) getZones() ([]string, error) {
+	responseData, err := api.request("POST", "zone/_search", nil)
+	if err != nil {
+		return nil, err
+	}
+
+	var responseObject JSONResponseDataZone
+	if err := json.Unmarshal(responseData, &responseObject); err != nil {
+		return nil, err
+	}
+
+	names := make([]string, 0, len(responseObject.Data))
+	for _, zone := range responseObject.Data {
+		names = append(names, zone.Origin)
+	}
+
+	return names, nil
+}
+
 func (api *autoDNSProvider) updateZone(domain string, resourceRecords []*ResourceRecord, nameServers []*models.Nameserver, zoneTTL uint32) error {
 	systemNameServer, err := api.findZoneSystemNameServer(domain)
-
 	if err != nil {
 		return err
 	}
 
-	zone, _ := api.getZone(domain)
+	zone, err2 := api.getZone(domain)
+
+	if err2 != nil {
+		return err2
+	}
 
 	zone.Origin = domain
 	zone.SystemNameServer = systemNameServer.Name
@@ -129,7 +224,7 @@ func (api *autoDNSProvider) updateZone(domain string, resourceRecords []*Resourc
 
 	zone.NameServers = append(zone.NameServers, nameServers...)
 
-	var _, putErr = api.request("PUT", "zone/"+domain+"/"+systemNameServer.Name, zone)
+	_, putErr := api.request("PUT", "zone/"+domain+"/"+systemNameServer.Name, zone)
 
 	if putErr != nil {
 		return putErr
@@ -137,3 +232,12 @@ func (api *autoDNSProvider) updateZone(domain string, resourceRecords []*Resourc
 
 	return nil
 }
+
+func (api *autoDNSProvider) updateDomain(name string, domain *Domain) error {
+	_, err := api.request("PUT", "domain/"+name, domain)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
