@@ -9,12 +9,13 @@ import (
 	"sort"
 	"strconv"
 	"strings"
+	"sync"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	"github.com/StackExchange/dnscontrol/v4/pkg/txtutil"
-	"github.com/StackExchange/dnscontrol/v4/providers"
 	dnsimpleapi "github.com/dnsimple/dnsimple-go/dnsimple"
 	"golang.org/x/oauth2"
 )
@@ -23,8 +24,9 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Can(),
+	providers.CanConcur:              providers.Can(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Cannot(),
@@ -41,12 +43,15 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
-	providers.RegisterRegistrarType("DNSIMPLE", newReg)
+	const providerName = "DNSIMPLE"
+	const providerMaintainer = "@onlyhavecans"
+	providers.RegisterRegistrarType(providerName, newReg)
 	fns := providers.DspFuncs{
 		Initializer:   newDsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("DNSIMPLE", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 const stateRegistered = "registered"
@@ -70,7 +75,15 @@ var nameServerSuffixes = []string{
 type dnsimpleProvider struct {
 	AccountToken string // The account access token
 	BaseURL      string // An alternate base URI
-	accountID    string // Account id cache
+
+	// We can have multiple _distinct_ versions of this struct, authenticated to
+	// different accounts, so _each_ version needs to be initialized just once.
+	// Note that sync.Once contains a reference to sync.noCopy so this will cause
+	// `go vet` to catch attempts to copy this outer provider struct.
+	onceFetchAccountID sync.Once
+
+	// This is protected under onceFetchAccountID so that this is fully safe concurrently.
+	accountID string // Account id cache
 }
 
 // GetNameservers returns the name servers for a domain.
@@ -138,7 +151,7 @@ func (c *dnsimpleProvider) GetZoneRecords(domain string, meta map[string]string)
 			if isQuotedTXT(r.Content) {
 				err = rec.PopulateFromStringFunc(r.Type, r.Content, domain, txtutil.ParseQuoted)
 			} else {
-				err = rec.SetTargetTXT(fmt.Sprintf("legacy: %s", r.Content))
+				err = rec.SetTargetTXT("legacy: " + r.Content)
 			}
 		default:
 			err = rec.PopulateFromString(r.Type, r.Content, domain)
@@ -157,17 +170,17 @@ func (c *dnsimpleProvider) GetZoneRecords(domain string, meta map[string]string)
 	return cleanedRecords, nil
 }
 
-func (c *dnsimpleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, error) {
+func (c *dnsimpleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, int, error) {
 	removeOtherApexNS(dc)
 
 	dnssecFixes, err := c.getDNSSECCorrections(dc)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
-	toReport, create, del, modify, err := diff.NewCompat(dc).IncrementalDiff(actual)
+	toReport, create, del, modify, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(actual)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
@@ -199,7 +212,7 @@ func (c *dnsimpleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, ac
 		})
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func removeApexNS(records models.Records) models.Records {
@@ -296,16 +309,22 @@ func (c *dnsimpleProvider) getClient() *dnsimpleapi.Client {
 }
 
 func (c *dnsimpleProvider) getAccountID() (string, error) {
-	if c.accountID == "" {
+	var onceErr error
+	c.onceFetchAccountID.Do(func() {
 		client := c.getClient()
 		whoamiResponse, err := client.Identity.Whoami(context.Background())
 		if err != nil {
-			return "", err
+			onceErr = err
+			return
 		}
 		if whoamiResponse.Data.User != nil && whoamiResponse.Data.Account == nil {
-			return "", fmt.Errorf("DNSimple token appears to be a user token. Please supply an account token")
+			onceErr = errors.New("DNSimple token appears to be a user token. Please supply an account token")
+			return
 		}
 		c.accountID = strconv.FormatInt(whoamiResponse.Data.Account.ID, 10)
+	})
+	if onceErr != nil {
+		return "", onceErr
 	}
 	return c.accountID, nil
 }
@@ -458,7 +477,6 @@ func (c *dnsimpleProvider) getNameservers(domainName string) ([]string, error) {
 	}
 
 	if domainResponse.Data.State == stateRegistered {
-
 		delegationResponse, err := client.Registrar.GetDomainDelegation(context.Background(), accountID, domainName)
 		if err != nil {
 			var errorResponse *dnsimpleapi.ErrorResponse
@@ -648,7 +666,7 @@ func newProvider(m map[string]string, _ json.RawMessage) (*dnsimpleProvider, err
 	api := &dnsimpleProvider{}
 	api.AccountToken = m["token"]
 	if api.AccountToken == "" {
-		return nil, fmt.Errorf("missing DNSimple token")
+		return nil, errors.New("missing DNSimple token")
 	}
 
 	if m["baseurl"] != "" {
@@ -721,12 +739,13 @@ func getTargetRecordPriority(rc *models.RecordConfig) int {
 
 // Compile the error messages returned by DNSimple's API into a single error message
 func compileAttributeErrors(err *dnsimpleapi.ErrorResponse) error {
-	message := fmt.Sprintf("%d %s", err.HTTPResponse.StatusCode, err.Message)
+	var message strings.Builder
+	message.WriteString(fmt.Sprintf("%d %s", err.HTTPResponse.StatusCode, err.Message))
 	for field, errors := range err.AttributeErrors {
 		e := strings.Join(errors, "& ")
-		message += fmt.Sprintf(": %s %s", field, e)
+		message.WriteString(fmt.Sprintf(": %s %s", field, e))
 	}
-	return fmt.Errorf(message)
+	return errors.New(message.String())
 }
 
 // Return true if the string ends in one of DNSimple's name server domains
