@@ -1,12 +1,13 @@
 package easyname
 
 import (
+	"errors"
 	"fmt"
 	"sort"
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
-	"github.com/StackExchange/dnscontrol/v4/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 )
 
 type easynameProvider struct {
@@ -19,18 +20,21 @@ type easynameProvider struct {
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
-	providers.CanConcur: providers.Cannot(),
+	providers.CanConcur: providers.Unimplemented(),
 }
 
 func init() {
-	providers.RegisterRegistrarType("EASYNAME", newEasyname, features)
+	const providerName = "EASYNAME"
+	const providerMaintainer = "@tresni"
+	providers.RegisterRegistrarType(providerName, newEasyname, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 func newEasyname(m map[string]string) (providers.Registrar, error) {
 	api := &easynameProvider{}
 
 	if m["email"] == "" || m["userid"] == "" || m["apikey"] == "" || m["authsalt"] == "" || m["signsalt"] == "" {
-		return nil, fmt.Errorf("missing easyname email, userid, apikey, authsalt and/or signsalt")
+		return nil, errors.New("missing easyname email, userid, apikey, authsalt and/or signsalt")
 	}
 
 	api.apikey, api.signSalt = m["apikey"], m["signsalt"]
