@@ -34,9 +34,10 @@ type gcoreRRSetExtended struct {
 	TTL     int                     `json:"ttl"`
 	Records []dnssdk.ResourceRecord `json:"resource_records"`
 	Filters []dnssdk.RecordFilter   `json:"filters"`
+	Meta    *gcoreMetadata          `json:"meta"`
 }
 
-func dnssdkDo(ctx context.Context, c *dnssdk.Client, apiKey string, method, uri string, bodyParams interface{}, dest interface{}) error {
+func dnssdkDo(ctx context.Context, c *dnssdk.Client, apiKey string, method, uri string, bodyParams any, dest any) error {
 	// Adapted from https://github.com/G-Core/gcore-dns-sdk-go/blob/main/client.go#L289
 	// No way to reflect a private method in Golang
 
@@ -64,7 +65,7 @@ func dnssdkDo(ctx context.Context, c *dnssdk.Client, apiKey string, method, uri
 	}
 
 	req.Header.Set("Content-Type", "application/json")
-	req.Header.Set("Authorization", fmt.Sprintf("APIKey %s", apiKey))
+	req.Header.Set("Authorization", "APIKey "+apiKey)
 	if c.UserAgent != "" {
 		req.Header.Set("User-Agent", c.UserAgent)
 	}
@@ -92,7 +93,7 @@ func dnssdkDo(ctx context.Context, c *dnssdk.Client, apiKey string, method, uri
 		return nil
 	}
 
-	// nolint: wrapcheck
+	//nolint: wrapcheck
 	return json.NewDecoder(resp.Body).Decode(dest)
 }
 
@@ -114,7 +115,7 @@ func (c *gcoreProvider) dnssdkRRSets(domain string) (gcoreRRSets, error) {
 
 func (c *gcoreProvider) dnssdkGetDNSSEC(domain string) (bool, error) {
 	var result gcoreZone
-	url := fmt.Sprintf("/v2/zones/%s", domain)
+	url := "/v2/zones/" + domain
 
 	err := dnssdkDo(c.ctx, c.provider, c.apiKey, http.MethodGet, url, nil, &result)
 	if err != nil {
