@@ -17,6 +17,7 @@ Settings from `creds.json`:
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"sort"
 	"strconv"
@@ -25,7 +26,7 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
-	"github.com/StackExchange/dnscontrol/v4/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	"github.com/miekg/dns/dnsutil"
 )
 
@@ -33,12 +34,15 @@ import (
 
 // init registers the provider to dnscontrol.
 func init() {
+	const providerName = "LOOPIA"
+	const providerMaintainer = "@systemcrash"
 	fns := providers.DspFuncs{
 		Initializer:   newDsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("LOOPIA", fns, features)
-	providers.RegisterRegistrarType("LOOPIA", newReg)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterRegistrarType(providerName, newReg)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // features declares which features and options are available.
@@ -46,20 +50,25 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Cannot(),
+	providers.CanConcur:              providers.Unimplemented(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAKAMAICDN:        providers.Cannot(),
 	providers.CanUseAlias:            providers.Cannot(),
 	providers.CanUseAzureAlias:       providers.Cannot(),
 	providers.CanUseCAA:              providers.Can(),
+	providers.CanUseDHCID:            providers.Cannot(), // Verified 2025-07-24
+	providers.CanUseDNSKEY:           providers.Cannot(), // Verified 2025-07-24
 	providers.CanUseDS:               providers.Cannot("Only supports DS records at the apex, only for .se and .nu domains; done automatically at back-end."),
 	providers.CanUseDSForChildren:    providers.Cannot(),
+	providers.CanUseHTTPS:            providers.Cannot(), // Verified 2025-07-24
 	providers.CanUseLOC:              providers.Can(),
 	providers.CanUseNAPTR:            providers.Can(),
 	providers.CanUsePTR:              providers.Cannot(),
 	providers.CanUseSOA:              providers.Cannot("💩"),
 	providers.CanUseSRV:              providers.Can(),
 	providers.CanUseSSHFP:            providers.Can(),
+	providers.CanUseSVCB:             providers.Cannot(), // Verified 2025-07-24
 	providers.CanUseTLSA:             providers.Can(),
 	providers.DocCreateDomains:       providers.Cannot("Can only manage domains registered through their service"),
 	providers.DocDualHost:            providers.Can(),
@@ -83,10 +92,10 @@ func newReg(conf map[string]string) (providers.Registrar, error) {
 // newHelper generates a handle.
 func newHelper(m map[string]string, _ json.RawMessage) (*APIClient, error) {
 	if m["username"] == "" {
-		return nil, fmt.Errorf("missing Loopia API username")
+		return nil, errors.New("missing Loopia API username")
 	}
 	if m["password"] == "" {
-		return nil, fmt.Errorf("missing Loopia API password")
+		return nil, errors.New("missing Loopia API password")
 	}
 
 	const booleanStringWarn = " setting as a 'string': 't', 'true', 'True' etc"
@@ -96,7 +105,7 @@ func newHelper(m map[string]string, _ json.RawMessage) (*APIClient, error) {
 	if m["modify_name_servers"] != "" { // optional
 		modifyNameServers, err = strconv.ParseBool(m["modify_name_servers"])
 		if err != nil {
-			return nil, fmt.Errorf("creds.json requires the modify_name_servers" + booleanStringWarn)
+			return nil, errors.New("creds.json requires the modify_name_servers" + booleanStringWarn)
 		}
 	}
 
@@ -104,15 +113,15 @@ func newHelper(m map[string]string, _ json.RawMessage) (*APIClient, error) {
 	if m["fetch_apex_ns_entries"] != "" { // optional
 		fetchApexNSEntries, err = strconv.ParseBool(m["fetch_apex_ns_entries"])
 		if err != nil {
-			return nil, fmt.Errorf("creds.json requires the fetch_apex_ns_entries" + booleanStringWarn)
+			return nil, errors.New("creds.json requires the fetch_apex_ns_entries" + booleanStringWarn)
 		}
 	}
 
 	dbg := false
-	if m["debug"] != "" { //debug is optional
+	if m["debug"] != "" { // debug is optional
 		dbg, err = strconv.ParseBool(m["debug"])
 		if err != nil {
-			return nil, fmt.Errorf("creds.json requires the debug" + booleanStringWarn)
+			return nil, errors.New("creds.json requires the debug" + booleanStringWarn)
 		}
 	}
 
@@ -130,7 +139,6 @@ func newHelper(m map[string]string, _ json.RawMessage) (*APIClient, error) {
 
 // ListZones lists the zones on this account.
 func (c *APIClient) ListZones() ([]string, error) {
-
 	listResp, err := c.getDomains()
 	if err != nil {
 		return nil, err
@@ -160,12 +168,11 @@ func (c *APIClient) ListZones() ([]string, error) {
 // GetZoneRecords gathers the DNS records and converts them to
 // dnscontrol's format.
 func (c *APIClient) GetZoneRecords(domain string, meta map[string]string) (models.Records, error) {
-
 	// Two approaches. One: get all SubDomains, and get their respective records
 	// simultaneously, or first get subdomains then fill each subdomain with its
 	// respective records on a subsequent pass.
 
-	//step 1: subdomains
+	// step 1: subdomains
 	// Get existing subdomains for a domain:
 	subdomains, err := c.GetSubDomains(domain)
 	if err != nil {
@@ -179,12 +186,12 @@ func (c *APIClient) GetZoneRecords(domain string, meta map[string]string) (model
 	// Convert them to DNScontrol's native format:
 	existingRecords := []*models.RecordConfig{}
 	for _, subdomain := range subdomains {
-		//here seems like a good place to get the records for a subdomain.
-		//fukn ballz tho: each subdomain requires one API call. 💩
+		// here seems like a good place to get the records for a subdomain.
+		// fukn ballz tho: each subdomain requires one API call. 💩
 		if c.Debug {
 			fmt.Printf("%s\n", subdomain)
 		}
-		//step 2: records for subdomains
+		// step 2: records for subdomains
 		// Get subdomain records:
 		subdomainrecords, err := c.getDomainRecords(domain, subdomain)
 		if err != nil {
@@ -192,16 +199,13 @@ func (c *APIClient) GetZoneRecords(domain string, meta map[string]string) (model
 		}
 
 		for _, subdRr := range subdomainrecords {
-
-			//Note: subdomain cannot be any of [.-_ ]
+			// Note: subdomain cannot be any of [.-_ ]
 			record, err := nativeToRecord(subdRr, domain, subdomain)
 			if err != nil {
 				return nil, err
 			}
 			existingRecords = append(existingRecords, record)
-
 		}
-
 	}
 
 	if c.Debug {
@@ -213,7 +217,7 @@ func (c *APIClient) GetZoneRecords(domain string, meta map[string]string) (model
 
 // PrepFoundRecords munges any records to make them compatible with
 // this provider. Usually this is a no-op.
-//func PrepFoundRecords(recs models.Records) models.Records {
+// func PrepFoundRecords(recs models.Records) models.Records {
 // If there are records that need to be modified, removed, etc. we
 // do it here.  Usually this is a no-op.
 //return recs
@@ -231,7 +235,7 @@ func PrepDesiredRecords(dc *models.DomainConfig) {
 		if rec.Type == "ALIAS" {
 			// Loopia does not support ALIAS.
 			// Therefore, we change this to a CNAME.
-			rec.Type = "CNAME"
+			rec.ChangeType("CNAME", dc.Name)
 		}
 		if rec.TTL < 300 {
 			/* you can submit TTL lower than 300 but the dig results are normalized to 300 */
@@ -267,8 +271,7 @@ func gatherAffectedLabels(groups map[models.RecordKey][]string) (labels map[stri
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (c *APIClient) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
-
+func (c *APIClient) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	if c.Debug {
 		debugRecords("GenerateZoneRecordsCorrections input:\n", existingRecords)
 	}
@@ -277,16 +280,16 @@ func (c *APIClient) GetZoneRecordsCorrections(dc *models.DomainConfig, existingR
 
 	var keysToUpdate map[models.RecordKey][]string
 	differ := diff.NewCompat(dc)
-	toReport, create, del, modify, err := differ.IncrementalDiff(existingRecords)
+	toReport, create, del, modify, actualChangeCount, err := differ.IncrementalDiff(existingRecords)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
 
-	keysToUpdate, _, err = differ.ChangedGroups(existingRecords)
+	keysToUpdate, _, _, err = differ.ChangedGroups(existingRecords)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	for _, d := range create {
@@ -327,7 +330,7 @@ func (c *APIClient) GetZoneRecordsCorrections(dc *models.DomainConfig, existingR
 				subdomain := dnsutil.TrimDomainName(fqdn, dc.Name)
 				if d.Existing.NameFQDN == fqdn && d.Existing.Name == subdomain {
 					// fmt.Printf("fqdn extinct wtf: %s\n", fqdn)
-					//deletion is a member of fqdn. skip its deletion (otherwise extra API call and its error)
+					// deletion is a member of fqdn. skip its deletion (otherwise extra API call and its error)
 					skip = true
 				}
 			}
@@ -354,20 +357,20 @@ func (c *APIClient) GetZoneRecordsCorrections(dc *models.DomainConfig, existingR
 		corrections = append(corrections, &models.Correction{
 			Msg: d.String(),
 			F: func() error {
-				//weird BUG: if we provide d.Desired.Name, instead of 'subdomain',
-				//all change records get assigned a single subdomain, common across all change records.
+				// weird BUG: if we provide d.Desired.Name, instead of 'subdomain',
+				// all change records get assigned a single subdomain, common across all change records.
 				// return c.UpdateRecordSimulate(dc.Name, subdomain, zrec)
 				return c.UpdateRecord(dc.Name, subdomain, zrec)
 			},
 		})
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 // debugRecords prints a list of RecordConfig.
 func debugRecords(note string, recs []*models.RecordConfig) {
-	printer.Debugf(note)
+	printer.Debugf("%s", note)
 	for k, v := range recs {
 		printer.Printf("   %v: %v %v %v %v\n", k, v.GetLabel(), v.Type, v.TTL, v.GetTargetCombined())
 	}
@@ -389,7 +392,6 @@ func (c *APIClient) GetNameservers(domain string) ([]*models.Nameserver, error)
 
 // GetRegistrarCorrections returns a list of corrections for this registrar.
 func (c *APIClient) GetRegistrarCorrections(dc *models.DomainConfig) ([]*models.Correction, error) {
-
 	existingNs, err := c.GetDomainNS(dc.Name)
 	if err != nil {
 		return nil, err
@@ -408,7 +410,8 @@ func (c *APIClient) GetRegistrarCorrections(dc *models.DomainConfig) ([]*models.
 				F: func() (err error) {
 					// err = c.UpdateNameServers(dc.Name, desiredNs)
 					return
-				}},
+				},
+			},
 		}, nil
 	}
 	return nil, nil
