@@ -2,12 +2,13 @@ package netlify
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
-	"github.com/StackExchange/dnscontrol/v4/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	"github.com/miekg/dns"
 )
 
@@ -15,8 +16,9 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Cannot(),
+	providers.CanConcur:              providers.Can(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Cannot(),
@@ -33,13 +35,16 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "NETLIFY"
+	const providerMaintainer = "@SphericalKat"
 	fns := providers.DspFuncs{
 		Initializer:   newNetlify,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("NETLIFY", fns, features)
-	providers.RegisterCustomRecordType("NETLIFY", "NETLIFY", "")
-	providers.RegisterCustomRecordType("NETLIFYv6", "NETLIFY", "")
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterCustomRecordType(providerName, providerName, "")
+	providers.RegisterCustomRecordType("NETLIFYv6", providerName, "")
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 type netlifyProvider struct {
@@ -51,7 +56,7 @@ func newNetlify(m map[string]string, message json.RawMessage) (providers.DNSServ
 	api := &netlifyProvider{}
 	api.apiToken = m["token"]
 	if api.apiToken == "" {
-		return nil, fmt.Errorf("missing Netlify personal access token")
+		return nil, errors.New("missing Netlify personal access token")
 	}
 
 	api.accountSlug = m["slug"]
@@ -80,7 +85,7 @@ func (n *netlifyProvider) getZone(domain string) (*dnsZone, error) {
 		}
 	}
 
-	return nil, fmt.Errorf("no zones found for this domain")
+	return nil, errors.New("no zones found for this domain")
 }
 
 func (n *netlifyProvider) GetZoneRecords(domain string, meta map[string]string) (models.Records, error) {
@@ -157,17 +162,17 @@ func (n *netlifyProvider) ListZones() ([]string, error) {
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (n *netlifyProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, records models.Records) ([]*models.Correction, error) {
-	toReport, create, del, modify, err := diff.NewCompat(dc).IncrementalDiff(records)
+func (n *netlifyProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, records models.Records) ([]*models.Correction, int, error) {
+	toReport, create, del, modify, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(records)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
 
 	zone, err := n.getZone(dc.Name)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	// Deletes first so changing type works etc.
@@ -211,7 +216,7 @@ func (n *netlifyProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, rec
 		corrections = append(corrections, corr)
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func toReq(rc *models.RecordConfig) *dnsRecordCreate {
