@@ -3,23 +3,26 @@ package oracle
 import (
 	"context"
 	"encoding/json"
+	"errors"
+	"net/http"
 	"strings"
 	"time"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
-	"github.com/StackExchange/dnscontrol/v4/providers"
-	"github.com/oracle/oci-go-sdk/v32/common"
-	"github.com/oracle/oci-go-sdk/v32/dns"
-	"github.com/oracle/oci-go-sdk/v32/example/helpers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
+	"github.com/oracle/oci-go-sdk/v65/common"
+	"github.com/oracle/oci-go-sdk/v65/dns"
+	"github.com/oracle/oci-go-sdk/v65/example/helpers"
 )
 
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
+	providers.CanConcur:              providers.Unimplemented(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Cannot(), // should be supported, but getting 500s in tests
@@ -35,11 +38,14 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "ORACLE"
+	const providerMaintainer = "@kallsyms"
 	fns := providers.DspFuncs{
 		Initializer:   New,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("ORACLE", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 type oracleProvider struct {
@@ -61,6 +67,12 @@ func New(settings map[string]string, _ json.RawMessage) (providers.DNSServicePro
 		return nil, err
 	}
 
+	// Set default retry policy to handle 429 automatically
+	defaultRetryPolicy := common.DefaultRetryPolicy()
+	client.SetCustomClientConfiguration(common.CustomClientConfiguration{
+		RetryPolicy: &defaultRetryPolicy,
+	})
+
 	return &oracleProvider{
 		client:      client,
 		compartment: settings["compartment"],
@@ -83,11 +95,26 @@ func (o *oracleProvider) ListZones() ([]string, error) {
 	for i, zone := range listResp.Items {
 		zones[i] = *zone.Name
 	}
+
+	for listResp.OpcNextPage != nil {
+		listResp, err = o.client.ListZones(ctx, dns.ListZonesRequest{
+			CompartmentId: &o.compartment,
+			Page:          listResp.OpcNextPage,
+		})
+		if err != nil {
+			return nil, err
+		}
+
+		for _, zone := range listResp.Items {
+			zones = append(zones, *zone.Name)
+		}
+	}
+
 	return zones, nil
 }
 
 // EnsureZoneExists creates a zone if it does not exist
-func (o *oracleProvider) EnsureZoneExists(domain string) error {
+func (o *oracleProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
 	ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
 	defer cancel()
 
@@ -98,7 +125,7 @@ func (o *oracleProvider) EnsureZoneExists(domain string) error {
 	if err == nil {
 		return nil
 	}
-	if getResp.RawResponse.StatusCode != 404 {
+	if getResp.RawResponse.StatusCode != http.StatusNotFound {
 		return err
 	}
 
@@ -120,7 +147,7 @@ func (o *oracleProvider) EnsureZoneExists(domain string) error {
 		}
 		return true
 	}
-	_, err = o.client.GetZone(ctx, dns.GetZoneRequest{
+	getResp, err = o.client.GetZone(ctx, dns.GetZoneRequest{
 		ZoneNameOrId:    &domain,
 		CompartmentId:   &o.compartment,
 		RequestMetadata: helpers.GetRequestMetadataWithCustomizedRetryPolicy(pollUntilAvailable),
@@ -146,7 +173,17 @@ func (o *oracleProvider) GetNameservers(domain string) ([]*models.Nameserver, er
 		nss[i] = *ns.Hostname
 	}
 
-	return models.ToNameservers(nss)
+	nssNoStrip, err := models.ToNameservers(nss)
+	if err != nil {
+		nssStrip, err := models.ToNameserversStripTD(nss)
+		if err != nil {
+			return nil, errors.New("could not determine if trailing dots should be stripped or not")
+		}
+
+		return nssStrip, nil
+	}
+
+	return nssNoStrip, nil
 }
 
 func (o *oracleProvider) GetZoneRecords(zone string, meta map[string]string) (models.Records, error) {
@@ -204,7 +241,7 @@ func (o *oracleProvider) GetZoneRecords(zone string, meta map[string]string) (mo
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (o *oracleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
+func (o *oracleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	var err error
 
 	// Ensure we don't emit changes for attempted modification of built-in apex NSs
@@ -219,15 +256,15 @@ func (o *oracleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, exis
 			continue
 		}
 
-		if rec.TTL != 86400 {
+		if rec.GetLabel() == "@" && rec.TTL != 86400 {
 			printer.Warnf("Oracle Cloud forces TTL=86400 for NS records. Ignoring configured TTL of %d for %s\n", rec.TTL, recNS)
 			rec.TTL = 86400
 		}
 	}
 
-	toReport, create, dels, modify, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
+	toReport, create, dels, modify, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
@@ -240,46 +277,43 @@ func (o *oracleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, exis
 		for any size zone.
 	*/
 
-	desc := ""
+	var desc strings.Builder
 	createRecords := models.Records{}
 	deleteRecords := models.Records{}
 
 	if len(create) > 0 {
 		for _, rec := range create {
 			createRecords = append(createRecords, rec.Desired)
-			desc += rec.String() + "\n"
+			desc.WriteString(rec.String() + "\n")
 		}
-		desc = desc[:len(desc)-1]
 	}
 
 	if len(dels) > 0 {
 		for _, rec := range dels {
 			deleteRecords = append(deleteRecords, rec.Existing)
-			desc += rec.String() + "\n"
+			desc.WriteString(rec.String() + "\n")
 		}
-		desc = desc[:len(desc)-1]
 	}
 
 	if len(modify) > 0 {
 		for _, rec := range modify {
 			createRecords = append(createRecords, rec.Desired)
 			deleteRecords = append(deleteRecords, rec.Existing)
-			desc += rec.String() + "\n"
+			desc.WriteString(rec.String() + "\n")
 		}
-		desc = desc[:len(desc)-1]
 	}
 
 	// There were corrections. Send them as one big batch:
 	if len(createRecords) > 0 || len(deleteRecords) > 0 {
 		corrections = append(corrections, &models.Correction{
-			Msg: desc,
+			Msg: desc.String(),
 			F: func() error {
 				return o.patch(createRecords, deleteRecords, dc.Name)
 			},
 		})
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func (o *oracleProvider) patch(createRecords, deleteRecords models.Records, domain string) error {
@@ -301,11 +335,9 @@ func (o *oracleProvider) patch(createRecords, deleteRecords models.Records, doma
 	}
 
 	for batchStart := 0; batchStart < len(ops); batchStart += 100 {
-		batchEnd := batchStart + 100
-		if batchEnd > len(ops) {
-			batchEnd = len(ops)
-		}
+		batchEnd := min(batchStart+100, len(ops))
 		patchReq.Items = ops[batchStart:batchEnd]
+
 		_, err := o.client.PatchZoneRecords(ctx, patchReq)
 		if err != nil {
 			return err
