@@ -7,13 +7,12 @@ import (
 	"fmt"
 	"strings"
 
-	"golang.org/x/net/idna"
-	"golang.org/x/oauth2"
-
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
-	"github.com/StackExchange/dnscontrol/v4/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	"github.com/vultr/govultr/v2"
+	"golang.org/x/net/idna"
+	"golang.org/x/oauth2"
 )
 
 /*
@@ -29,7 +28,7 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanConcur:              providers.Unimplemented(),
 	providers.CanUseAlias:            providers.Cannot(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseLOC:              providers.Cannot(),
@@ -42,11 +41,14 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "VULTR"
+	const providerMaintainer = "@pgaskin"
 	fns := providers.DspFuncs{
 		Initializer:   NewProvider,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("VULTR", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // vultrProvider represents the Vultr DNSServiceProvider.
@@ -65,7 +67,7 @@ var defaultNS = []string{
 func NewProvider(m map[string]string, metadata json.RawMessage) (providers.DNSServiceProvider, error) {
 	token := m["token"]
 	if token == "" {
-		return nil, fmt.Errorf("missing Vultr API token")
+		return nil, errors.New("missing Vultr API token")
 	}
 
 	config := &oauth2.Config{}
@@ -112,27 +114,28 @@ func (api *vultrProvider) GetZoneRecords(domain string, meta map[string]string)
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (api *vultrProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, curRecords models.Records) ([]*models.Correction, error) {
+func (api *vultrProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, curRecords models.Records) ([]*models.Correction, int, error) {
 	var corrections []*models.Correction
 
 	for _, rec := range dc.Records {
 		switch rec.Type { // #rtype_variations
-		case "ALIAS", "MX", "NS", "CNAME", "PTR", "SRV", "URL", "URL301", "FRAME", "R53_ALIAS", "NS1_URLFWD", "AKAMAICDN", "CLOUDNS_WR":
+		case "ALIAS", "MX", "NS", "CNAME", "PTR", "SRV", "URL", "URL301", "FRAME", "R53_ALIAS", "AKAMAICDN", "CLOUDNS_WR":
 			// These rtypes are hostnames, therefore need to be converted (unlike, for example, an AAAA record)
 			t, err := idna.ToUnicode(rec.GetTargetField())
 			if err != nil {
-				return nil, err
+				return nil, 0, err
+			}
+			if err := rec.SetTarget(t); err != nil {
+				return nil, 0, err
 			}
-			rec.SetTarget(t)
 		default:
 			// Nothing to do.
 		}
 	}
 
-	changes, err := diff2.ByRecord(curRecords, dc, nil)
-
+	changes, actualChangeCount, err := diff2.ByRecord(curRecords, dc, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	for _, change := range changes {
@@ -169,7 +172,7 @@ func (api *vultrProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, cur
 		}
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 // GetNameservers gets the Vultr nameservers for a domain
@@ -178,7 +181,7 @@ func (api *vultrProvider) GetNameservers(domain string) ([]*models.Nameserver, e
 }
 
 // EnsureZoneExists creates a zone if it does not exist
-func (api *vultrProvider) EnsureZoneExists(domain string) error {
+func (api *vultrProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
 	if ok, err := api.isDomainInAccount(domain); err != nil {
 		return err
 	} else if ok {
@@ -227,7 +230,7 @@ func toRecordConfig(domain string, r govultr.DomainRecord) (*models.RecordConfig
 	rc.SetLabel(r.Name, domain)
 
 	switch rtype := r.Type; rtype {
-	case "ALIAS", "MX", "NS", "CNAME", "PTR", "SRV", "URL", "URL301", "FRAME", "R53_ALIAS", "NS1_URLFWD", "AKAMAICDN", "CLOUDNS_WR":
+	case "ALIAS", "MX", "NS", "CNAME", "PTR", "SRV", "URL", "URL301", "FRAME", "R53_ALIAS", "AKAMAICDN", "CLOUDNS_WR":
 		var err error
 		data, err = idna.ToUnicode(data)
 		if err != nil {
